/* File %M%, version %R%.%L%, released  %E% at %U%
   retrieved by SCCS %D% at %T%

%% Program to compute simulated 2-bit samples of noise + tone.

:: Offline diagnostic

HISTORY:
090327 GIL fix time series plot, find spectral peaks after averaging all data
090326 GIL complete time ordering fix, and average more spectra
090325 GIL add options for 1, 2 or 4 data streams
070907 GIL fix up event finding
070906 GIL make time axis in nano-seconds
070724 GIL make more average spectra (A/B and Long/short)
070721 GIL by default the bandwidth is 500 MHz
070717 GIL optionally jump data in 4 byte chunks
070716 GIL add arguments for plots, process many files
070715 GIL select a set of plots
070714 GIL creates a first good set of plots
070713 GIL initial version

DESCRIPTION:
eventFft() generate a report summary for an input list of events.
noise and an test tone.
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "MATHCNST.H"
#include "STDDEFS.H"

extern void realft(float *data, long n, long isign);
extern double drand48();    /* random number generator, uniform on [0,1). */
extern long atol(char *);
extern long strtol(char *, char **, int base);
extern void four1(float data[], unsigned long nn, int isign);
extern char * quickPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n,  double * xs, 
		  double * y1s, char * y1Label,
		  double * y2s, char * y2Label,
                  double * y3s, char * y3Label,
		  double * y4s, char * y4Label);
extern char * stripWhite( char *);
extern char * calcAveRmsRange( double a[], long aStart, long aEnd, 
       long bStart, long bEnd, double * aveOut, double * rmsOut);

/* internals */
double gasdev();
#define BASE 16
#define TWOTO16 32768
#define TWOTO32 4294967296
#define MAXSAMPLES  TWOTO16
#define MAXLINE     1000
#define MAXPLOTS    30
#define MAXFILES    20000
long debug = FALSE;

long next2( long i) 
{ long nFft = i;

  if (nFft < 32)
    nFft = 16;
  else if (nFft < 64)
    nFft = 32;
  else if (nFft < 128)
    nFft = 64;
  else if (nFft < 256)
    nFft = 128;
  else if (nFft < 512)
    nFft = 256;
  else if (nFft < 1024)
    nFft = 512;
  else if (nFft < 2048)
    nFft = 1024;
  else if (nFft < 4096)
    nFft = 2048;
  else if (nFft < 8192) 
    nFft = 4096;
  else 
    nFft = 8192;
  return( nFft);
} /* end of next2() */

int main (int argc, char * argv[]) 
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* orderTone() creates two files containing VSOP I and Q data streams        */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char fileName[1000] = "", record[1000] = "", dateTimeStr[1000] = "",
    title[1000] = "", * fileNames[MAXFILES], logName[1000] = "event.log",
    outDir[1000] = "/tmp/", fullName[1000] = "", inDir[1000] = "./",
    plotName[1000] = "", label[1000] = "", aLabel[1000] = "H", 
    bLabel[1000]="V",
    *gnuCmds = "set terminal postscript color \"Helvetica\" 18";
  unsigned long evtClkFast = 0, evtClkSlow = 0, 
    synClkFast = 0, synClkSlow = 0, rtClkFast = 0, rtClkSlow = 0, sample = 0;
  long i = 0, j = 0, nFft = 0, iAPeak = 0, count = 0, doJump = FALSE,
    eStart = 0, eEnd = 0, nEvent = 0, iBPeak = 0, nWindow = 32, nSearch=1000,
    iAFreq = 0, iBFreq = 0, doTime = FALSE, tStart = 0, tEnd = 0, nFiles = 0,
    doBandSpec = FALSE, doEventTime = FALSE , doEventSpec = FALSE, n0 = 0,
    nChan = 0, iFile = 0, nSpec = 0, doAveSpec = FALSE, nEChan = 0, nStep = 0,
    eventDurations[MAXFILES], nPlot = 0, nEventFft = 0, nEventSpec = 0,
    isLong = FALSE, iEAFreq = 0, iEBFreq = 0, nLongSpec = 0, iLAFreq = 0, 
    doLongSpec = FALSE, iLBFreq = 0, nFourier = 0, n1 = 0, n2 = 0, n3 = 0,
    nColumns = 1, k = 0, d = 0;
  double bFreq = 0, eFreq = 950, dFreq = 0, cFreq = 950., xSigma = 0, 
    as[MAXSAMPLES], bs[MAXSAMPLES], dStep = 0, cs[MAXSAMPLES],
    eventASpecs[MAXSAMPLES], eventBSpecs[MAXSAMPLES], tFactor = 1,
    fs[MAXSAMPLES], ts[MAXSAMPLES], eventTimes[MAXFILES], 
    eventAPeaks[MAXFILES], eventBPeaks[MAXFILES], 
    eventARms[MAXFILES], eventBRms[MAXFILES],
    eas[MAXSAMPLES], ebs[MAXSAMPLES], aAve = 0, aRms = 0, aPeak = 0,
    bPeak = 0, nSigma = 6., bAve, bRms, fAPeak = 0, fBPeak = 0, 
    aveASpecs[MAXSAMPLES], aveBSpecs[MAXSAMPLES], fLAPeak = 0, fLBPeak = 0,
    aveEventASpec[MAXSAMPLES], aveEventBSpec[MAXSAMPLES], fEAPeak = 0, 
    aveLongASpec[MAXSAMPLES], aveLongBSpec[MAXSAMPLES], fEBPeak = 0,
    aSpecs[MAXSAMPLES], bSpecs[MAXSAMPLES], sas[MAXSAMPLES], sbs[MAXSAMPLES],
    cSpecs[MAXSAMPLES], dSpecs[MAXSAMPLES];
  float 
    pas[MAXSAMPLES], pbs[MAXSAMPLES], fftA[MAXSAMPLES], fftB[MAXSAMPLES],
    fftC[MAXSAMPLES], fftD[MAXSAMPLES];
  FILE * pFile = NULL, * pLog = NULL;

  if (argc < 2) {
    printf(" eventFft: summarize and plot a series of captured events\n");
    printf(" eventFft: usage\n");
    printf(" eventFft [-1 -2 -4][-a] [-b <freqMHz>] [-e <freqMHz>] [-c <clockMHz>] [-z] <eventFileName1> [.. <eventFileNameN>]\n");
    printf(" Where\n");
    printf(" -1       plot 1 data stream (combine two columns, default)\n");
    printf(" -2       plot 2 columns separately\n");
    printf(" -4       plot 4 columns separately, if present\n");
    printf(" -a       optionally plot all plots\n");
    printf(" -b <freqMHz> optionally set the begin frequency\n");
    printf(" -e <freqMHz> optionally set the end frequency\n");
    printf(" -f       optionally plot the event spectrum\n");
    printf(" -g       optionally plot the event time series\n");
    printf(" -i <inDir>   optionally set input directory\n");
    printf(" -j       optionally jump byte order of four input data values\n");
    printf(" -l <labelString> optionally add name of event log file\n");
    printf("                  default /tmp/event.log\n");
    printf(" -m <nSamples>    optionally set number of channels in band pass spectrum\n"); 
    printf(" -n <nEvent>      optionally set number of channels in event spectrum (default 256)\n"); 
    printf(" -o <inDir>    optionally set output directory (default /tmp)\n"); 
    printf(" -s       optionally plot the bandpass spectrum before event\n");
    printf(" -t       optionally plot the entire time series\n");
    printf(" -x <xSigma>   optionally eXclude events below <xSigma>\n");
    printf(" -z            optionally compute the average spectrum\n");
    printf("           ----- Glen Langston, 2009 March 26\n");
    return(-1);
  }

  i = 1;
  while (i < argc) {                     /* for all input arguments */
    if (strncmp(argv[i],"-A",2) == 0 ||  /* if plot all plots */
        strncmp(argv[i],"-a",2) == 0) { 
      doTime = TRUE;
      doBandSpec = TRUE;
      doEventTime = TRUE;
      doEventSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-B",2) == 0 ||  /* set begin freq */
             strncmp(argv[i],"-b",2) == 0) {
       sscanf(argv[i+1], "%le", &bFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-C",2) == 0 ||  /* set clock freq */
             strncmp(argv[i],"-c",2) == 0) {
       sscanf(argv[i+1], "%le", &cFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-D",2) == 0 ||  /* offset in data streams */
             strncmp(argv[i],"-d",2) == 0) {
       sscanf(argv[i+1], "%ld", &d); 
       fprintf( stderr, "Using data skew value of %ld samples\n", d);
       i = i + 2;
    }
    else if (strncmp(argv[i],"-E",2) == 0 ||  /* set end freq */
             strncmp(argv[i],"-e",2) == 0) {
       sscanf(argv[i+1], "%le", &eFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-F",2) == 0 ||  /* if plot Event Spectrum */
	     strncmp(argv[i],"-f",2) == 0) { 
      doEventSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-G",2) == 0 ||  /* if plot Event Time Series */
	     strncmp(argv[i],"-g",2) == 0) { 
      doEventTime = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-I",2) == 0 ||  /* if in directory */
	     strncmp(argv[i],"-i",2) == 0) { 
      strcpy( inDir, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-J",2) == 0 ||  /* if jumping samples */
	     strncmp(argv[i],"-j",2) == 0) { 
      doJump = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-K",2) == 0 ||  /* if show RFI spectra */
	     strncmp(argv[i],"-k",2) == 0) { 
      doLongSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-L",2) == 0 ||  /* if label string */
	     strncmp(argv[i],"-l",2) == 0) { 
      strcpy( label, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-M",2) == 0 ||  /* if fixing band FFt length */
	     strncmp(argv[i],"-m",2) == 0) { 
      sscanf( argv[i+1], "%ld", &nEventFft);
      nEventFft = next2( nEventFft);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-N",2) == 0 ||  /* if fixing event FFt length */
	     strncmp(argv[i],"-n",2) == 0) { 
      sscanf( argv[i+1], "%ld", &nEventFft);
      nEventFft = next2( nEventFft);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-O",2) == 0 ||  /* if output directory */
	     strncmp(argv[i],"-o",2) == 0) { 
      strcpy( outDir, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-S",2) == 0 ||  /* if plot Spectrum */
	     strncmp(argv[i],"-s",2) == 0) { 
      doBandSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-T",2) == 0 ||  /* if plot Time Seris */
        strncmp(argv[i],"-t",2) == 0) { 
      doTime = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-X",2) == 0 ||  /* if plot average Spectrum */
        strncmp(argv[i],"-x",2) == 0) { 
      sscanf(argv[i+1], "%lf", &xSigma);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-1",2) == 0) { /* plot only one column */
      nColumns = 1;
      i++;
    }
    else if (strncmp(argv[i],"-2",2) == 0) { /* plot two columns */
      nColumns = 2;
      i++;
    }
    else if (strncmp(argv[i],"-4",2) == 0) { /* plot four columns */
      nColumns = 4;
      i++;
    }
    else {
      fileNames[nFiles] = argv[i];
      nFiles++;
      i++;
    }
  } /* end while all arguments */

  if ((doTime == FALSE) && (doBandSpec == FALSE) && 
      (doEventTime == FALSE) && (doEventSpec = FALSE) && (doAveSpec == FALSE))
    doTime = TRUE;

  if (outDir[ strlen(outDir)-1] != '/')
    strcat( outDir, "/");

  if (strlen( label) > 1) 
    sprintf( logName, "%s.log", label);

  strcpy( fullName, outDir);
  if (logName[0] == '/') 
    strcpy( fullName, logName);
  else 
    strcat( fullName, logName);

  pLog = fopen( fullName, "w+");
  if (pLog == NULL) {
    fprintf( stderr, "Error openning event log file: %s\n", fullName);
  }

  /* for all input files */
  for (iFile = 0; iFile < nFiles; iFile++) {
    /* prepare to open file */
    strcpy( fullName, inDir);
    if (fullName[ strlen(fullName)-1] != '/')
      strcat( fullName, "/");
    strcpy( fileName, fileNames[iFile]);

    if (fileName[0] == '/') 
      strcpy( fullName, fileName);
    else
      strcat( fullName, fileName);

    /* in case there is no date time entry in file, use file name */
    j = 0; /* strip out path from the file name */
    for (i = 0; i < strlen( fileName); i++) {
      if (fileName[i] == '/')                /* if / then start of file name */
	j = 0;
      else {
	dateTimeStr[j] = fileName[i];
	j++;
      }
    } /* end for all characters */
    dateTimeStr[j] = EOS;

    for (i = 0; i < strlen( dateTimeStr) - 1; i++)  /* remove file extension */
      if (dateTimeStr[i] == '.' && dateTimeStr[i+1] == 't')
	dateTimeStr[i] = EOS;

    pFile = fopen( fullName, "r");
    if (pFile == NULL) {
      fprintf( stderr, "Error openning event file: %s\n", fullName);
      break;
    }

    if (debug) 
      fprintf( stderr, "Event file: %s -> %s -> %s\n", 
	       fullName, fileName, dateTimeStr);

    if (eFreq != bFreq) {
      if (eFreq > bFreq) 
	cFreq = eFreq - bFreq;
      else
	cFreq = bFreq - eFreq;
    }

    if (cFreq <= 1.)                /* default clock rate is 320 MHz */
      cFreq = 320.0;

  /* read an event File: format example 
# EVT: 5590 2392617869 2007-07-14T04:00:02.68
0        0.023438    0.031250
1        0.078125    0.007812
2        0.085938   -0.039062
3        0.070312   -0.062500
...
  */
    count = 0;                             /* count data samples */
    for (j=0; j < MAXSAMPLES; j++) {        /* for all bytes of file */
      if (fgets(record,132,pFile) == NULL) 
	break;
      stripWhite( record);
      if (strlen(record) <  1)              /* ignore blank lines */
	continue;
      /* look for an event record */
      if ((record[0] == '#') && (count == 0)) {
	/* # EVT: 5590 2392617869 2007-07-14T04:00:02.68 */
	for (i = 5; i < MAXLINE; i++) {
	  if (record[i-5] == '#' && record[i-4] == ' ' &&
	      record[i-3] == 'E' && record[i-2] == 'V' &&
	      record[i-1] == 'T' && record[i] == ':') {
	    sscanf( &record[i+1], "%lu %lu %s\n", &evtClkSlow, &evtClkFast,
		    dateTimeStr);
	    break;
	  }
	  /*# SYN: 5 1297166302 RT: 5 1325592158 EVT: 5 1297166332 DATE: 2007-07-18T21:19:20.15
	   */
	  if (record[i-3] == 'S' && record[i-2] == 'Y' &&
	      record[i-1] == 'N' && record[i] == ':') {
	    sscanf( &record[i+1], "%lu %lu\n", &synClkSlow, &synClkFast);
	  }
	  if (record[i-3] == ' ' && record[i-2] == 'R' &&
	      record[i-1] == 'T' && record[i] == ':') {
	    sscanf( &record[i+1], "%lu %lu\n", &rtClkSlow, &rtClkFast);
	  }
	  if (record[i-3] == 'E' && record[i-2] == 'V' &&
	      record[i-1] == 'T' && record[i] == ':') {
	    sscanf( &record[i+1], "%lu %lu\n", &evtClkSlow, &evtClkFast);
	  }
	  if (record[i-4] == 'D' &&record[i-3] == 'T' && record[i-2] == 'A' &&
	      record[i-1] == 'E' && record[i] == ':') {
	    sscanf( &record[i+1], "%s\n", dateTimeStr);
	  }
	} /* end for all characters in record */
      } /* end if a timing event record */
      /* if a comment, print and skip */
      if ((record[0] == '#') || (record[0] == '!') ||
	(record[0] == '$') || (record[0] == ';')) {
	fprintf( stderr, "%s\n", record);
	continue;
      } /* end if a comment */
      sscanf( record, "%lu %g %g", &sample, &pas[count], &pbs[count]);
      count++;
    } /* end for all input event samples */

    if (doJump) {                        /* if bytes are flipped */
      for (j=0; j < count; j = j+4) {    /* for all bytes of file */
	as[j]   = pas[j+3];              /* re-arrange data out of long */
	as[j+1] = pas[j+2];
	as[j+2] = pas[j+1];
	as[j+3] = pas[j];
	bs[j]   = pbs[j+3];
	bs[j+1] = pbs[j+2];
	bs[j+2] = pbs[j+1];
	bs[j+3] = pbs[j];
      } 
      for (j=0; j < count; j++) {        /* for all bytes of file */
	pas[j] = as[j];                  /* transfer back */
	pbs[j] = bs[j];
      }
    }
    else {                               /* else normal data order */
      for (j=0; j < count; j++) {        /* for all bytes of file */
        as[j] = pas[j];                  /* transfer */
        bs[j] = pbs[j];
      }
    } /* end else normal data order */
       
    fclose(pFile);                     /* done reading file */

    if (nColumns == 1) {               /* if only one data stream */
      i = 0;
      cs[0] = cs[1] = cs[2] = cs[3] = 0;   /* need to zero incase of skew */
      bs[count] = bs[count+1] = bs[count+2] = bs[count+3] = 0;
      for (j=0; j < count; j = j +4) {      /* transfer back */
        cs[i] = as[j+0];
	i++;
        cs[i] = bs[j+0+d];
	i++;
        cs[i] = as[j+1];
	i++;
        cs[i] = bs[j+1+d];
	i++;
        cs[i] = as[j+2];
	i++;
        cs[i] = bs[j+2+d];
	i++;
        cs[i] = as[j+3];
	i++;
        cs[i] = bs[j+3+d];
	i++;
      }
      count = i;
      for (j=0; j < count; j++)        /* transfer back */
	as[j] = cs[j];
      if (debug)
	fprintf( stderr, "Processing a single data stream of %ld samples\n",
	       count);
    } /* end if merging data streams */

    if (nColumns == 1) 
      tFactor = 1000./cFreq;
    else
      tFactor = 2000./cFreq;

    i = (count/2);                     /* set center sample */
    for (j=0; j < count; j++)          /* for all time values */
      ts[j] = (j - i)*tFactor;

    count = count - 4;                 /* get rid of noise at end of data */

    aPeak = bPeak = 0;
    for (i = 0; i < count; i++) {      /* look for peak in data */
      if (fabs( as[i]) > aPeak) {
	aPeak = fabs(as[i]);
	iAPeak = i;
      }
      if (fabs( bs[i]) > bPeak) {
	bPeak = fabs(bs[i]);
	iBPeak = i;
      }
    } /* end for all input samples */

    if (count < 3000)  /* skip this file */
      continue;

    calcAveRmsRange( as, 0, 1, 2, count, &aAve, &aRms);
    calcAveRmsRange( bs, 0, 1, 2, count, &bAve, &bRms);

    if (((aPeak/aRms) < xSigma) && ((bPeak/bRms) < xSigma)) {
      fprintf( stderr, "Weak Peak, not averaging or logging %s\n", 
	       dateTimeStr);      continue;
    }

    tEnd = eEnd = count;
    nSearch = count/10.;
    tStart = eStart = nSearch;
    /* now find events by searching for points above nSigma */
    for (i = nSearch; i < count-nSearch; i++) {
      if(as[i] > nSigma*aRms || as[i] < -nSigma*aRms) {
	if (eStart == nSearch) 
	  eStart = i-nWindow;
	eEnd = i+nWindow;
	if (tStart == nSearch) 
	  tStart = i;
	tEnd = i+1;
      } /* if an A time series peak found */
      if (nColumns > 1) {
	if(bs[i] > nSigma*bRms || bs[i] < -nSigma*bRms) {
	  if (eStart == nSearch) 
	    eStart = i-nWindow;
	  eEnd = i+nWindow;
	  if (tStart == nSearch) 
	    tStart = i;
	  tEnd = i+1;
	}
      } /* if more than one data stream */
    } /* end for all input samples */

    nEvent = next2( eEnd-eStart);     /* must have power of 2 for FFT */
    if (nEvent > 128)                 /* can not have too big an event */
      nEvent = 256;
    if (nEventFft > 0)
      nEvent = nEventFft; 
    eEnd = eStart + nEvent;
    if ((nFft <= 0) || (nFft > (count/5)))
      nFft = next2(count / 5);

    /* record event parameters */
    eventDurations[iFile] = tEnd-tStart; 

    eventTimes[iFile] = (evtClkSlow*(unsigned long)TWOTO32) + evtClkFast;
    eventAPeaks[iFile] = as[iAPeak]; 
    eventBPeaks[iFile] = bs[iBPeak]; 
    eventARms[iFile] = aRms; 
    eventBRms[iFile] = bRms; 

    if (pLog) {
      if (nColumns > 1) 
	fprintf( pLog, "%7.4lf %6.3lf  %7.4lf %6.3lf %5ld ",
	       eventAPeaks[iFile], eventARms[iFile], 
	       eventBPeaks[iFile], eventBRms[iFile], eventDurations[iFile]);
      else
	fprintf( pLog, "%7.4lf %6.3lf  %5ld ",
	       eventAPeaks[iFile], eventARms[iFile], eventDurations[iFile]);
    }

    fprintf( stderr, "%4ld Event %s at %5ld (nEvent:%4ld nFFt:%4ld)\n", 
	     count, dateTimeStr, tEnd-tStart, nEvent, nFft);

    fprintf( stderr, "A[%5ld]: %7.4lf;  Ave %7.4lf, Rms %6.3lf\n", 
	     iAPeak, as[iAPeak], aAve, aRms);
    if (nColumns > 1) 
      fprintf( stderr, "B[%5ld]: %7.4lf;  Ave %7.4lf, Rms %6.3lf\n", 
	     iBPeak, bs[iBPeak], bAve, bRms);

    /* transform to frequency domain */
    /*    four1( &pas[-1], nFft, (unsigned long)1);  */
    /* now flip order for FT, excluding the zero lag point  */
    j = 0;
    if (nColumns == 1) {
      n0 = (count - nFft - 1);       /* get data after end of event */
      n1 = 100;                      /* get data after beginning */
      n2 = n1 + nFft;
      n3 = n0 - nFft;
      for(i=0; i<nFft; i++) {
	fftA[j] =as[n1+i] ;          /* move from very edge of data */
	fftB[j] =as[n0+i];           /* make a second spectrum, from end */
	fftC[j] =as[n2+i];           /* make a second spectrum, from end */
	fftD[j] =as[n2+i];           /* make a second spectrum, from end */
	j++;
	fftA[j] = 0;
	fftB[j] = 0;
	fftC[j] = 0;
	fftD[j] = 0;
	j++;
      }
    } /* end if only one data stream */
    else {
      for(i=0; i<nFft; i++) {
	fftA[j] = as[i];
	fftB[j] = bs[i];
	j++;
	fftA[j] = 0;
	fftB[j] = 0;
	j++;
      }
    } /* end else more than one data stream */
    four1( fftA-1, nFft, (unsigned long)1); 
    four1( fftB-1, nFft, (unsigned long)1); 
    four1( fftC-1, nFft, (unsigned long)1); 
    four1( fftD-1, nFft, (unsigned long)1); 

    /* compute power spectrum */
    nChan = 0;
    for (i = 0; i < nFft; i = i + 2) {
      aSpecs[nChan] = fftA[i]*fftA[i] + fftA[i+1]*fftA[i+1];
      if (aSpecs[nChan] > 0) 
	aSpecs[nChan] = sqrt( aSpecs[nChan]);
      bSpecs[nChan] = fftB[i]*fftB[i] + fftB[i+1]*fftB[i+1];
      if (bSpecs[nChan] > 0) 
	bSpecs[nChan] = sqrt( bSpecs[nChan]);
      cSpecs[nChan] = fftC[i]*fftC[i] + fftC[i+1]*fftC[i+1];
      if (bSpecs[nChan] > 0) 
	cSpecs[nChan] = sqrt( cSpecs[nChan]);
      dSpecs[nChan] = fftD[i]*fftD[i] + fftD[i+1]*fftD[i+1];
      if (dSpecs[nChan] > 0) 
	dSpecs[nChan] = sqrt( dSpecs[nChan]);
      nChan++;
    } /* end for all data in FFt */

    if (nColumns == 1) {            /* if only 1 data stream, average 2 spec */
      for (i = 0; i < nChan; i++) {
	aSpecs[i] = 0.25 * (aSpecs[i] + bSpecs[i]+cSpecs[i]+dSpecs[i]);
	bSpecs[i] = 0.5 * (bSpecs[i] + dSpecs[i]);
      }
    } /* end if averaging several spectra */
    aSpecs[0] = 0;
    aSpecs[nChan-1] = 0;
    bSpecs[0] = 0;
    bSpecs[nChan-1] = 0;

    dFreq = (eFreq - bFreq)/nChan;  /* calculate frequency interval */
    for (i = 0; i < nChan; i++) 
      fs[i] = bFreq + (i*dFreq);

    fAPeak = 0;                              /* find frequency peak */
    fBPeak = 0;                              /* find frequency peak */
    for (i = 1; i < nChan-1; i++) {
      if (fabs( aSpecs[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( aSpecs[i]);
      }
      if (fabs( bSpecs[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( bSpecs[i]);
      }
    } /* end for all peak frequency channels */

    iEAFreq = iAFreq;
    iEBFreq = iBFreq;
    if (nSpec == 0) {                /* if first spectrum, init */
      for (i = 0; i < nChan; i++) {
	aveASpecs[i] = aSpecs[i];
	aveBSpecs[i] = bSpecs[i];
      }
    }
    else {                         /* else sum for later average */
      for (i = 0; i < nChan; i++) { 
	aveASpecs[i] = aveASpecs[i] + aSpecs[i];
	aveBSpecs[i] = aveBSpecs[i] + bSpecs[i];
      }
    }
    nSpec++;                       /* count number of spectra averaged */

    if (pLog) {
      if (nColumns > 1) 
	fprintf( pLog, "%12.6lf %12.6lf ", fs[iAFreq], fs[iBFreq]);
      else
	fprintf( pLog, "%12.6lf ", fs[iAFreq]);
    }
    if ((tEnd-tStart) > 128 ) /* if event is many samples */
      isLong = TRUE;
    else
      isLong = FALSE;

    if (nFourier == 0)
      nFourier = next2(nEvent);

    j = 0;
    /* select samples for Event spectra */
    for (i = eStart; (i < (eStart + nFourier)) && (i < count); i++) {
      fftA[j] = as[i];
      fftB[j] = bs[i];
      j++;
      fftA[j] = 0;
      fftB[j] = 0;
      j++;
      eas[i-eStart] = as[i];     /* save event time series */
      ebs[i-eStart] = bs[i];
    }

    if (debug)
      fprintf( stderr, "nEvent %ld (%ld to %ld)\n", nFourier, eStart, eEnd);
    
    /* transform to frequency domain */
    four1( fftA-1, nFourier, (unsigned long)1); 
    four1( fftB-1, nFourier, (unsigned long)1); 

    nEChan = 0;
    /* compute power spectrum */
    for (i = 0; i < nFourier; i = i + 2) {
      sas[nEChan] = fftA[i]*fftA[i] + fftA[i+1]*fftA[i+1];
      if (as[nEChan] > 0) 
	sas[nEChan] = sqrt( sas[nEChan]);
      sbs[nEChan] = fftB[i]*fftB[i] + fftB[i+1]*fftB[i+1];
      if (sbs[nEChan] > 0) 
	sbs[nEChan] = sqrt( sbs[nEChan]);
      nEChan++;
    } /* end for all data in FFt */

    nStep = nChan/nEChan;                  /* nStep is power of two */
    k = 0;
    dStep = nStep - 1.;
    sas[nFourier] = sas[nFourier-1];
    sbs[nFourier] = sbs[nFourier-1];       /* copy last point */
    for (i = 0; i < nEChan; i++) {
      for (j = 0; j < nStep; j++) {
	eventASpecs[k] = (((dStep-j)*sas[i]) + (j*sas[i+1]))/dStep;
	eventBSpecs[k] = (((dStep-j)*sbs[i]) + (j*sbs[i+1]))/dStep;
	k++;
      }
    }
    if (debug)
      fprintf( stderr, "Interpolation: nChan=%ld nEventChan=%ld\n",
	     nChan, nEChan);
      
    fAPeak = fBPeak = 0;
    for (i = 0; i < nChan; i++) {
      if (fabs( eventASpecs[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( eventASpecs[i]);
      }
      if (fabs( eventBSpecs[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( eventBSpecs[i]);
      }
    } /* end for all peak frequency channels */

    if (isLong) {
      if (nLongSpec == 0) {         /* if time to initialize the average */
	for (i = 0; i < nChan; i++) {
	  aveLongASpec[i] = eventASpecs[i];
	  aveLongBSpec[i] = eventBSpecs[i];
	}
      }
      else {
	for (i = 0; i < nChan; i++) {
	  aveLongASpec[i] = aveLongASpec[i] + eventASpecs[i];
	  aveLongBSpec[i] = aveLongBSpec[i] + eventBSpecs[i];
	}
      }
      nLongSpec++;
    } /* end if long RFI */
    else {
      if (nEventSpec == 0) {         /* if time to initialize the average */
	for (i = 0; i < nChan; i++) {
	  aveEventASpec[i] = eventASpecs[i];
	  aveEventBSpec[i] = eventBSpecs[i];
	}
      }
      else {
	for (i = 0; i < nChan; i++) {
	  aveEventASpec[i] = aveEventASpec[i] + eventBSpecs[i];
	  aveEventBSpec[i] = aveEventBSpec[i] + eventBSpecs[i];
	}
      }
      nEventSpec++;
    } /* end if an event */

    if (pLog) 
      fprintf( pLog, "%12.6lf %12.6lf %s\n", fs[iAFreq], fs[iBFreq],
	       dateTimeStr);
  } /* end for all files */

  if (nEventSpec > 0) {                       /* if any spectra produced */
    for (i = 0; i < nChan; i++) {
      aveEventASpec[i] = aveEventASpec[i]/nEventSpec;
      aveEventBSpec[i] = aveEventBSpec[i]/nEventSpec;
    }
    aveEventASpec[0] = aveEventASpec[nChan-1] = 0;   /* zero end channels */
    aveEventBSpec[0] = aveEventBSpec[nChan-1] = 0;   /* zero end channels */

    fEAPeak = fEBPeak = 0;
    for (i = 1; i < nChan-1; i++) {
      if (fabs( aveEventASpec[i]) > fEAPeak) {
	iEAFreq = i;
	fEAPeak = fabs( aveEventASpec[i]);
      }
      if (fabs( aveEventBSpec[i]) > fEBPeak) {
	iEBFreq = i;
	fEBPeak = fabs( aveEventBSpec[i]);
      }
    } /* end for all peak frequency channels */
  } /* end if event spectra produced  */

  if (nLongSpec > 0) { 
    for (i = 0; i < nChan; i++) {
      aveLongASpec[i] = aveLongASpec[i]/nLongSpec;
      aveLongBSpec[i] = aveLongBSpec[i]/nLongSpec;
    }
    
    aveLongASpec[0] = aveLongASpec[nEChan-1] = 0;
    aveLongBSpec[0] = aveLongBSpec[nEChan-1] = 0;
    
    fLAPeak = fLBPeak = 0;
    for (i = 1; i < nChan-1; i++) {
      if (fabs( aveLongASpec[i]) > fLAPeak) {
	iLAFreq = i;
	fLAPeak = fabs( aveLongASpec[i]);
      }
      if (fabs( aveLongBSpec[i]) > fLBPeak) {
	iLBFreq = i;
	fLBPeak = fabs( aveLongBSpec[i]);
      }
    } /* end for all peak frequency channels */
  } /* end if any long events found */

  if (nEventSpec <= 0) {        /* if no short events, use long event freq */
    if (nLongSpec > 0) {
      iEAFreq = iLAFreq;
      iEBFreq = iLBFreq;
    }
    else
      fprintf( stderr, "No Events found\n");
  } /* end if no short events found */
  else if (nLongSpec <= 0) {
    if (nEventSpec > 0) {
      iLAFreq = iEAFreq;
      iLBFreq = iEBFreq;
    }
  }

  if (nSpec > 0) {                       /* if any spectra produced */
    fAPeak = fBPeak = 0;
    for (i = 0; i < nChan; i++)          /* normalize the spectra */
      aveASpecs[i] = aveASpecs[i]/nSpec;
    for (i = 0; i < nChan; i++)          /* normalize the spectra */
      aveBSpecs[i] = aveBSpecs[i]/nSpec;

    aveASpecs[0] = aveASpecs[nChan-1] = 0;   /* zero end channels */
    aveBSpecs[0] = aveBSpecs[nChan-1] = 0;   /* zero end channels */
    fAPeak = fBPeak = 0;
    for (i = 1; i < nChan-1; i++) {
      if (fabs( aveASpecs[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( aveASpecs[i]);
      }
      if (fabs( aveBSpecs[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( aveBSpecs[i]);
      }
    } /* end for all peak frequency channels */
  } /* end if any spectra accumulated */

  if (doTime) {
    if (nColumns == 1)
      sprintf( title, "%s Time Series %s: Peaks %.3lf MHz, %ld Samples", 
	       label, dateTimeStr, fs[iAFreq], tEnd-tStart);
    else
      sprintf( title, "%s Time Series %s: Peaks %.3lf,%.3lf MHz, %ld Samples", 
	       label, dateTimeStr, fs[iAFreq], fs[iBFreq], tEnd-tStart);
  
    sprintf( plotName, "%s%sTime",outDir,dateTimeStr);
    if (nColumns == 1) 
      quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Nano-seconds", "Voltage",
		 count, ts, as, "I", NULL, NULL, NULL, NULL, NULL, NULL);
    else 
      quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Nano-seconds", "Voltage",
		 count, ts, as, "H", bs, "V", NULL, NULL, NULL, NULL);
    nPlot++;
  } /* end else plot time series */
	       
  if (doEventTime) {
    sprintf( title, "%s Event Time Series: %s, %ld Samples", 
	     label, dateTimeStr, tEnd-tStart);
    
    sprintf( plotName, "%s%sEvent",outDir,dateTimeStr);
    if (nColumns == 1) 
      quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Nano-Seconds", "Voltage",
		 nEvent, &ts[eStart], eas, "I", NULL, NULL,
		 NULL, NULL, NULL, NULL);
    else
      quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Nano-Seconds", "Voltage",
	       nEvent, &ts[eStart], eas, "H", ebs, "V", 
	       NULL, NULL, NULL, NULL);
    nPlot++;
  } /* end if producing event time series */

  if (doBandSpec || doEventSpec) {
    if (nSpec > 1) 
      sprintf( title, "%s Average Band Pass+Event",
	     label); 
    else
      sprintf( title, "%s Band Pass+Event Spectra %s", 
	     label, dateTimeStr); 

    sprintf( plotName, "%s%sFft",outDir,dateTimeStr);

    if (nColumns == 1) {
      sprintf( aLabel, "Band, Peak %.3lf MHz", fs[iAFreq]);
      sprintf( bLabel, "Event, Peak %.3lf MHz", fs[iEAFreq]);
      if (nEventSpec <= 0)
	quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",
		 nChan, fs, aveASpecs, aLabel, aveLongASpec, bLabel, 
		 NULL, NULL, NULL, NULL);
      else
	quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",
		 nChan, fs, aveASpecs, aLabel, aveEventASpec, bLabel, 
		 NULL, NULL, NULL, NULL);
    }
    else {
      sprintf( aLabel, "Band H, Peak %.3lf MHz", fs[iAFreq]);
      sprintf( bLabel, "Band V, Peak %.3lf MHz", fs[iBFreq]);
      if (nEventSpec <= 0)
	quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",
		 nChan, fs, aveASpecs, aLabel, aveBSpecs, bLabel, 
		 aveLongASpec, "Event H", aveLongBSpec, "Event V"); 
      else
	quickPlot( plotName, (nPlot < MAXPLOTS), gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",
		 nChan, fs, aveASpecs, aLabel, aveBSpecs, bLabel, 
		 aveEventASpec, "Event H", aveEventBSpec, "Event V"); 
    }
    nPlot++;
  } /* end else more than 1 data column */

  if (doEventSpec) {
    sprintf( title, "%s Average Long Event Spectrum: Peaks %.3lf,%.3lf MHz", 
	   label, fs[iLAFreq], fs[iLBFreq]);
  
    sprintf( plotName, "%s%sAveLongSpec",outDir,label);
    if (nColumns == 1) {
      if (nEventSpec <= 0) 
	quickPlot( plotName, TRUE, gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveLongASpec, "Ave A", 
		 NULL, NULL, NULL, NULL, NULL, NULL);
      else
	quickPlot( plotName, TRUE, gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveEventASpec, "Ave A", 
		 NULL, NULL, NULL, NULL, NULL, NULL);
    }
    else {
      sprintf( aLabel, "Band H, Peak %.3lf MHz", fs[iEAFreq]);
      sprintf( bLabel, "Band V, Peak %.3lf MHz", fs[iEBFreq]);
      quickPlot( plotName, TRUE, gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveLongASpec, aLabel, 
		 aveLongBSpec, bLabel, NULL, NULL, NULL, NULL);
    }
    if (nColumns == 1) 
      quickPlot( plotName, TRUE, gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveEventASpec, "Ave A", 
		 NULL, NULL, NULL, NULL, NULL, NULL);
    else 
      quickPlot( plotName, TRUE, gnuCmds, title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveEventASpec, "Ave A", 
		 aveEventBSpec, "Ave B", NULL, NULL, NULL, NULL);
  } /* end if an event found */

  fprintf( stderr, "# %ld Total Events, %ld short strong events, %ld long strong\n",
	     nSpec, nEventSpec, nLongSpec);

  if (pLog) 
    fprintf( pLog, "# %ld Total Events, %ld short events, %ld long\n",
	     nSpec, nEventSpec, nLongSpec);
  fclose(pLog);

  return(0);
} /* end of eventFft */

