/* File eventFilter.c, version 1.1, released  09/03/26 at 09:26:12
   retrieved by SCCS 09/03/26 at 09:26:12

%% Program to compute simulated 2-bit samples of noise + tone.

:: Offline diagnostic

HISTORY:
070905 GIL initial version based on eventFind.c
070831 GIL reject spectra from average if FFT peak is near a radar altimeter
070805 GIL revised version
070724 GIL make more average spectra (A/B and Long/short)
070721 GIL by default the bandwidth is 500 MHz
070717 GIL optionally jump data in 4 byte chunks
070716 GIL add arguments for plots, process many files
070715 GIL select a set of plots
070714 GIL creates a first good set of plots
070713 GIL initial version

DESCRIPTION:
eventFilter communicates with the ibob and filters out RFI 
from average spectra
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "EVENT.H"
#include "MATHCNST.H"
#include "STDDEFS.H"

extern void realft(float *data, long n, long isign);
extern double drand48();    /* random number generator, uniform on [0,1). */
extern long atol(char *);
extern long strtol(char *, char **, int base);
extern void four1(float data[], unsigned long nn, int isign);
extern char * quickPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n,  double * xs, 
		  double * y1s, char * y1Label,
		  double * y2s, char * y2Label,
                  double * y3s, char * y3Label,
		  double * y4s, char * y4Label);
extern char * stripWhite( char *);
extern char * calcAveRmsRange( double a[], long aStart, long aEnd, 
       long bStart, long bEnd, double * aveOut, double * rmsOut);
extern double medianSelect(unsigned long k, unsigned long n, double arr[]);
extern char * closeIbobSocket();
char * getNextEvent( struct EVENTDATA * pEvent);

/* internals */
double gasdev();
#define BASE 16
#define TWOTO16 65536
#define TWOTO32 4294967296
#define MAXSAMPLES  TWOTO16
#define MAXLINE     1000
#define MAXPLOTS    30
#define MAXFILES    20000
long debug = FALSE;

long next2( long i) 
{ long nFft = i;

  if (nFft < 32)
    nFft = 16;
  else if (nFft < 64)
    nFft = 32;
  else if (nFft < 128)
    nFft = 64;
  else if (nFft < 256)
    nFft = 128;
  else if (nFft < 512)
    nFft = 256;
  else if (nFft < 1024)
    nFft = 512;
  else if (nFft < 2048)
    nFft = 1024;
  else 
    nFft = 2048;
  return( nFft);
} /* end of next2() */

#define NRMS 32

char * medianRms( double as[], long n, double * ave, double * rms)
{ unsigned long i = 0, nRms = NRMS, l = n/NRMS;
  double rmss[NRMS], aAve = 0, aRms = 0;

  for (i = 0; i < nRms; i++) { /* break samples into smaller chuncks */
    /* get rms for each chunck */
    calcAveRmsRange( as, i*l, (i+1)*l, i*l, i*l, &aAve, &aRms);
    rmss[i] = aRms;
  }
  *rms = medianSelect( nRms/2, nRms, &rmss[-1]);
  *ave = aAve;
  return( NULL);
} /* end of medianRms() */

int main (int argc, char * argv[]) 
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* orderTone() creates two files containing VSOP I and Q data streams        */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char fileName[1000] = "", record[1000] = "", dateTimeStr[1000] = "",
    title[1000] = "", * fileNames[MAXFILES], logName[1000] = "event.log",
    outDir[1000] = "/tmp/", fullName[1000] = "", inDir[1000] = "./",
    plotName[1000] = "", label[1000] = "";
  unsigned long evtClkFast = 0, evtClkSlow = 0, 
    synClkFast = 0, synClkSlow = 0, rtClkFast = 0, rtClkSlow = 0, sample = 0;
  long i = 0, j = 0, nFft = 2048, iAPeak = 0, count = 0, doJump = FALSE,
    eStart = 0, eEnd = 0, nEvent = 0, iBPeak = 0, nWindow = 32, 
    iAFreq = 0, iBFreq = 0, doTime = FALSE, tStart = 0, tEnd = 0, 
    nFiles = 999999,
    doBandSpec = FALSE, doEventTime = FALSE , doEventSpec = FALSE,
    nChan = 0, iFile = 0, nSpec = 0, doAveSpec = FALSE, nEChan = 0, 
    eventDurations[MAXFILES], nPlot = 0, nEventFft = 0, nEventSpec = 0,
    isLong = FALSE, iEAFreq = 0, iEBFreq = 0, nLongSpec = 0, iLAFreq = 0, 
    notMuchData = FALSE, doLongSpec = FALSE, iLBFreq = 0, nFourier = 0,
    nA = 0, nB = 0, nHigh = 0, nLow = 0, aLow = 0, bLow = 0, 
    doRfiFlag = FALSE, doRfiDelete = FALSE;
  double bFreq = 1600, eFreq = 800, dFreq = 0, cFreq = 1600., xSigma = 0, 
    as[MAXSAMPLES], bs[MAXSAMPLES], cs[MAXSAMPLES], es[MAXSAMPLES], 
    fs[MAXSAMPLES], ts[MAXSAMPLES], eventTimes[MAXFILES], rfiLevel = 20.,
    sas[MAXSAMPLES], sbs[MAXSAMPLES], eventAPeaks[MAXFILES], updateRate = 0.,
    eventBPeaks[MAXFILES], eventARms[MAXFILES], eventBRms[MAXFILES],
    eas[MAXSAMPLES], ebs[MAXSAMPLES], aRms = 0, aPeak = 0, aAve = 0, bAve = 0,
    bPeak = 0, nSigma = 5., bRms = 0, fAPeak = 0, fBPeak = 0, 
    aveASpec[MAXSAMPLES], aveBSpec[MAXSAMPLES], fLAPeak = 0, fLBPeak = 0,
    aveEventASpec[MAXSAMPLES], aveEventBSpec[MAXSAMPLES], fEAPeak = 0, 
    aveLongASpec[MAXSAMPLES], aveLongBSpec[MAXSAMPLES], fEBPeak = 0;
  float 
    pas[MAXSAMPLES], pbs[MAXSAMPLES], fftA[MAXSAMPLES], fftB[MAXSAMPLES];
  struct EVENTDATA event;
  FILE * pFile = NULL, * pLog = NULL;

  if (argc < 2) {
    printf(" eventFilter: summarize and plot a series of captured events\n");
    printf(" eventFilter: usage\n");
    printf(" eventFilter [-a] [-b <freqMHz>] [-e <freqMHz>] [-c <clockMHz>] [-z] <eventFileName1> [.. <eventFileNameN>]\n");
    printf(" Where\n");
    printf(" -a       optionally plot all plots\n");
    printf(" -b <freqMHz> optionally set the begin frequency\n");
    printf(" -e <freqMHz> optionally set the end frequency\n");
    printf(" -f       optionally plot the event spectrum\n");
    printf(" -g       optionally plot the event time series\n");
    printf(" -i <inDir>   optionally set input directory\n");
    printf(" -j       optionally jump byte order of four input data values\n");
    printf(" -n <nSamples> optionally set event fft length (ie 16,32,64 or ...)\n"); 
    printf(" -o <inDir>   optionally set output directory (default /tmp)\n"); 
    printf(" -r <rfiLevel> optionally flag data with RFI peak exceeding level\n");
    printf(" -s       optionally plot the bandpass spectrum before event\n");
    printf(" -t       optionally plot the entire time series\n");
    printf(" -x <xSigma> optionally eXclude events below <xSigma>\n");
    printf(" -z       optionally compute the average spectrum\n");
    printf("           ----- Glen Langston, 2007 August 5\n");
    return(-1);
  }

  i = 1;
  while (i < argc) {                     /* for all input arguments */
    if (strncmp(argv[i],"-A",2) == 0 ||  /* if plot all plots */
        strncmp(argv[i],"-a",2) == 0) { 
      doTime = TRUE;
      doBandSpec = TRUE;
      doEventTime = TRUE;
      doEventSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-B",2) == 0 ||  /* set begin freq */
             strncmp(argv[i],"-b",2) == 0) {
       sscanf(argv[i+1], "%le", &bFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-C",2) == 0 ||  /* set clock freq */
             strncmp(argv[i],"-c",2) == 0) {
       sscanf(argv[i+1], "%le", &cFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-D",2) == 0 ||  /* delete flagged events */
             strncmp(argv[i],"-d",2) == 0) {
       doRfiDelete = TRUE;
       i++;
    }
    else if (strncmp(argv[i],"-E",2) == 0 ||  /* set end freq */
             strncmp(argv[i],"-e",2) == 0) {
       sscanf(argv[i+1], "%le", &eFreq); 
       i = i + 2;
    }
    else if (strncmp(argv[i],"-F",2) == 0 ||  /* if plot Event Spectrum */
	     strncmp(argv[i],"-f",2) == 0) { 
      doEventSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-G",2) == 0 ||  /* if plot Event Time Series */
	     strncmp(argv[i],"-g",2) == 0) { 
      doEventTime = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-I",2) == 0 ||  /* if in directory */
	     strncmp(argv[i],"-i",2) == 0) { 
      strcpy( inDir, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-J",2) == 0 ||  /* if jumping samples */
	     strncmp(argv[i],"-j",2) == 0) { 
      doJump = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-K",2) == 0 ||  /* if show RFI spectra */
	     strncmp(argv[i],"-k",2) == 0) { 
      doLongSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-L",2) == 0 ||  /* if label string */
	     strncmp(argv[i],"-l",2) == 0) { 
      strcpy( label, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-N",2) == 0 ||  /* if fixing event FFt length */
	     strncmp(argv[i],"-n",2) == 0) { 
      sscanf( argv[i+1], "%ld", &nEventFft);
      nEventFft = next2( nEventFft);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-O",2) == 0 ||  /* if output directory */
	     strncmp(argv[i],"-o",2) == 0) { 
      strcpy( outDir, argv[i+1]);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-R",2) == 0 ||  /* if rfi Flag */
	     strncmp(argv[i],"-r",2) == 0) { 
      sscanf( argv[i+1], "%lf", &rfiLevel);
      fprintf( stderr, "Flagging RFI intensity > %lf\n", rfiLevel);
      doRfiFlag = TRUE;
      i = i + 2;
    }
    else if (strncmp(argv[i],"-S",2) == 0 ||  /* if plot Spectrum */
	     strncmp(argv[i],"-s",2) == 0) { 
      doBandSpec = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-T",2) == 0 ||  /* if plot Time Seris */
        strncmp(argv[i],"-t",2) == 0) { 
      doTime = TRUE;
      i++;
    }
    else if (strncmp(argv[i],"-U",2) == 0 ||  /* if plot Time Seris */
        strncmp(argv[i],"-u",2) == 0) { 
      sscanf( argv[i+1], "%lf", &updateRate);
      fprintf( stderr, "Averaging Spectra for %lf seconds\n", updateRate);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-X",2) == 0 ||  /* if plot average Spectrum */
        strncmp(argv[i],"-x",2) == 0) { 
      sscanf(argv[i+1], "%lf", &xSigma);
      i = i + 2;
    }
    else if (strncmp(argv[i],"-Z",2) == 0 ||  /* if plot average Spectrum */
        strncmp(argv[i],"-z",2) == 0) { 
      doAveSpec = TRUE;
      i++;
    }
  } /* end while all arguments */

  if ((doTime == FALSE) && (doBandSpec == FALSE) && 
      (doEventTime == FALSE) && (doEventSpec = FALSE) && (doAveSpec == FALSE))
    doTime = TRUE;

  if (outDir[ strlen(outDir)-1] != '/')
    strcat( outDir, "/");

  if (strlen( label) > 1) 
    sprintf( logName, "%s.log", label);

  strcpy( fullName, outDir);
  if (logName[0] == '/') 
    strcpy( fullName, logName);
  else 
    strcat( fullName, logName);

  pLog = fopen( fullName, "w+");
  if (pLog == NULL) {
    fprintf( stderr, "Error openning event log file: %s\n", fullName);
  }

  /* for all input files */
  for (iFile = 0; iFile < nFiles; iFile++) {

    if (debug) 
      fprintf( stderr, "Event file: %s -> %s -> %s\n", 
	       fullName, fileName, dateTimeStr);

    /* read an event File from Ibob */
    errMsg = getNextEvent( &event);    
    if (event.nData < 1) {
      /* usleep() */
      continue;
    }
    for (i = 0; i < event.nData; i++) {     /* look for peak in data */
      as[i] = event.as[i];
      bs[i] = event.bs[i];
    }
    count = event.nData;

    aPeak = bPeak = 0;
    for (i = 0; i < count; i++) {     /* look for peak in data */
      if (fabs( as[i]) > aPeak) {
	aPeak = fabs(as[i]);
	iAPeak = i;
      }
      if (fabs( bs[i]) > bPeak) {
	bPeak = fabs(bs[i]);
	iBPeak = i;
      }
    } /* end for all input samples */

    /* if not much data */
    if (count < 8000)  /* skip this file */
      continue;

    medianRms( as, count/2, &aAve, &aRms);
    medianRms( bs, count/2, &bAve, &bRms);

    /* remove sampler bias, this may also clean up FFT slightly */
    for (i = 0; i < count; i++) {
      as[i] -= aAve;
      bs[i] -= bAve;
    }

    if (((aPeak/aRms) < xSigma) && ((bPeak/bRms) < xSigma)) {
      fprintf( stderr, "Weak Peak, not averaging or logging %s\n", 
	       dateTimeStr);      continue;
    }

    tEnd = eEnd = count;
    tStart = eStart = nA = nB = 0;
    /* now find events by searching for points above nSigma */
    for (i = nWindow; i < count-nWindow; i++) {
      if(as[i] > nSigma*aRms || as[i] < -nSigma*aRms) {
	if (eStart == 0) 
	  eStart = i-nWindow;
	eEnd = i+nWindow;
	if (tStart == 0) 
	  tStart = i;
	tEnd = i+1;
        nA++;
      }
      else if (aLow == 0)
	aLow = i;
       
      if(bs[i] > nSigma*bRms || bs[i] < -nSigma*bRms) {
	if (eStart == 0) 
	  eStart = i-nWindow;
	eEnd = i+nWindow;
	if (tStart == 0) 
	  tStart = i;
	tEnd = i+1;
        nB++;
      }
      else if (bLow == 0)
	bLow = i;
    } /* end for all input samples */

    nEvent = next2( eEnd-eStart);     /* must have power of 2 for FFT */
    nEvent = 256;
    if (nEventFft > 0)
      nEvent = nEventFft; 
    eEnd = eStart + nEvent;
    nFft = next2(count / 2);

    if (nA > nB) 
      nHigh = nA;
    else
      nHigh = nB;

    /* record event parameters */
    eventDurations[iFile] = nHigh;

    eventTimes[iFile] = (event.eventClk[0]*TWOTO32) + event.eventClk;
    eventAPeaks[iFile] = as[iAPeak]; 
    eventBPeaks[iFile] = bs[iBPeak]; 
    eventARms[iFile] = aRms; 
    eventBRms[iFile] = bRms; 

    if (pLog) 
      fprintf( pLog, "%7.4lf %6.3lf  %7.4lf %6.3lf %5ld ",
	       eventAPeaks[iFile], eventARms[iFile], 
	       eventBPeaks[iFile], eventBRms[iFile], eventDurations[iFile]);
	       
    fprintf( stderr, "%4ld Event %s (nEvent: %4ld/%4ld nFFt:%4ld)\n", 
	     count, dateTimeStr, nHigh, nEvent, nFft);

    fprintf( stderr, "A Peak %4ld Sample: %7.4lf; Rms %6.3lf; Bias %6.5lf\n", 
	     iAPeak, as[iAPeak], aRms, aAve);
    fprintf( stderr, "B Peak %4ld Sample: %7.4lf; Rms %6.3lf; Bias %6.5lf\n", 
	     iBPeak, bs[iBPeak], bRms, bAve);

    /* transform to frequency domain */
    /*    four1( &pas[-1], nFft, (unsigned long)1);  */
    /* now flip order for FT, excluding the zero lag point  */
    j = 0;
    for(i=0; i<nFft; i++) {
      fftA[j] =pas[i+aLow];
      fftB[j] =pbs[i+bLow];
      j++;
      fftA[j] = 0;
      fftB[j] = 0;
      j++;
    }

    four1( fftA-1, nFft, (unsigned long)1); 
    four1( fftB-1, nFft, (unsigned long)1); 

    /* compute power spectrum */
    nChan = 0;
    for (i = 0; i < nFft; i = i + 2) {
      sas[nChan] = fftA[i]*fftA[i] + fftA[i+1]*fftA[i+1];
      if (sas[nChan] > 0) 
	sas[nChan] = sqrt( sas[nChan]);
      sbs[nChan] = fftB[i]*fftB[i] + fftB[i+1]*fftB[i+1];
      if (sbs[nChan] > 0) 
	sbs[nChan] = sqrt( sbs[nChan]);
      cs[nChan] = 0.5*(sas[nChan]+sbs[nChan]);
      nChan++;
    } /* end for all data in FFt */

    cs[0] = 0;
    sas[0] = 0;
    sas[nChan-1] = 0;
    sbs[0] = 0;
    sbs[nChan-1] = 0;

    dFreq = (eFreq - bFreq)/nChan;  /* calculate frequency interval */
    for (i = 0; i < nChan; i++) 
      fs[i] = bFreq + (i*dFreq);

    fAPeak = 0;                              /* find frequency peak */
    fBPeak = 0;                              /* find frequency peak */
    for (i = 1; i < nChan-1; i++) {
      if (fabs( sas[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( sas[i]);
      }
      if (fabs( sbs[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( sbs[i]);
      }
    } /* end for all peak frequency channels */

    if (pLog) 
      fprintf( pLog, "%12.6lf %12.6lf ", fs[iAFreq], fs[iBFreq]);

    if (doRfiFlag)                   /* if flagging RFI */
      if (fAPeak > rfiLevel || fBPeak > rfiLevel) {  /* if an RFI peak */
        fprintf( pLog, " Flagged                  %s\n", 
	       dateTimeStr);
	if (doRfiDelete)
	  remove( fileName);
	continue;
      } /* end if RFI peak found */

    if (doBandSpec) {
      sprintf( title, "%s Spectrum %s: Peaks %.3lf,%.3lf MHz", label, 
	       dateTimeStr, fs[iAFreq], fs[iBFreq]);

      sprintf( plotName, "%s%sBand",outDir,dateTimeStr);
      quickPlot( plotName, (nPlot < MAXPLOTS), "", title, "Frequency (MHz)", 
		 "Intensity",
		 nChan, fs, sas, "H", sbs, "V", NULL, NULL, NULL, NULL);
      nPlot++;
    }

    if (nSpec == 0) {                /* if first spectrum, init */
      for (i = 0; i < nChan; i++) {
	aveASpec[i] = sas[i];
	aveBSpec[i] = sbs[i];
      }
    }
    else {                         /* else sum for later average */
      for (i = 0; i < nChan; i++) { 
	aveASpec[i] = aveASpec[i] + sas[i];
	aveBSpec[i] = aveBSpec[i] + sbs[i];
      }
    }
    nSpec++;                       /* count number of spectra averaged */

    if (doTime) {
      sprintf( title, "%s Time Series %s: Peaks %.3lf,%.3lf MHz, %ld Samples", 
	       label, dateTimeStr, fs[iAFreq], fs[iBFreq], tEnd-tStart);
  
      for (i = 0; i < count; i++) 
	cs[i] = 0.5*(as[i]+bs[i]);
      
      sprintf( plotName, "%s%sTime",outDir,dateTimeStr);
      quickPlot( plotName, (nPlot < MAXPLOTS), "", title, 
		   "Samples", "Voltage",
		 count, ts, as, "H", bs, "V", NULL, NULL, NULL, NULL);
      nPlot++;
    } /* end else plot time series */
	       
    if (nHigh > 128 ) /* if event is many samples */
      isLong = TRUE;
    else
      isLong = FALSE;

    if (isLong) {
      nFourier = nFft;
      if (nEvent > count/2)
	notMuchData = TRUE;
      else 
	notMuchData = FALSE;
      if ((eStart + nFourier) > count)
	eStart = count - nFourier;
    }
    else {
      notMuchData = FALSE;
      nFourier = nEvent;
    }

    j = 0;
    for (i = eStart; (i < (eStart + nFourier)) && (i < count); i++) {
      fftA[j] = as[i];
      fftB[j] = bs[i];
      j++;
      fftA[j] = 0;
      fftB[j] = 0;
      j++;
      eas[i-eStart] = as[i];
      ebs[i-eStart] = bs[i];
    }

    if (debug)
      fprintf( stderr, "nEvent %ld (%ld to %ld)\n", nFourier, eStart, eEnd);
    
    /* transform to frequency domain */
    four1( fftA-1, nFourier, (unsigned long)1); 
    four1( fftB-1, nFourier, (unsigned long)1); 

    nEChan = 0;
    for (i = 0; i < nFourier; i = i + 2) {
      as[nEChan] = fftA[i]*fftA[i] + fftA[i+1]*fftA[i+1];
      if (as[nEChan] > 0) 
	as[nEChan] = sqrt( as[nEChan]);
      bs[nEChan] = fftB[i]*fftB[i] + fftB[i+1]*fftB[i+1];
      if (bs[nEChan] > 0) 
	bs[nEChan] = sqrt( bs[nEChan]);
      nEChan++;
    } /* end for all data in FFt */

    dFreq = (eFreq - bFreq)/nEChan;  /* calculate frequency interval */
    for (i = 0; i < nEChan; i++) 
      es[i] = bFreq + (i*dFreq);

    fAPeak = fBPeak = 0;
    for (i = 0; i < nEChan; i++) {
      if (fabs( as[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( as[i]);
      }
      if (fabs( bs[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( bs[i]);
      }
    } /* end for all peak frequency channels */

    if (isLong) {
      if (nLongSpec == 0) {         /* if time to initialize the average */
	for (i = 0; i < nEChan; i++) {
	  aveLongASpec[i] = as[i];
	  aveLongBSpec[i] = bs[i];
	}
      }
      else {
	for (i = 0; i < nEChan; i++) {
	  aveLongASpec[i] = aveLongASpec[i] + as[i];
	  aveLongBSpec[i] = aveLongBSpec[i] + bs[i];
	}
      }
      nLongSpec++;
    } /* end if long RFI */
    else {
      if (nEventSpec == 0) {         /* if time to initialize the average */
	for (i = 0; i < nEChan; i++) {
	  aveEventASpec[i] = as[i];
	  aveEventBSpec[i] = bs[i];
	}
      }
      else {
	for (i = 0; i < nEChan; i++) {
	  aveEventASpec[i] = aveEventASpec[i] + as[i];
	  aveEventBSpec[i] = aveEventBSpec[i] + bs[i];
	}
      }
      nEventSpec++;
    } /* end if an event */

    if (isLong && doRfiDelete) {                /* if flagging RFI */
      fprintf( pLog, " Flagged Long Event       %s\n", 
	       dateTimeStr);
      remove( fileName);
      continue;
    } /* end if RFI peak found */

    if (pLog) 
      fprintf( pLog, "%12.6lf %12.6lf %s\n", fs[iAFreq], fs[iBFreq],
	       dateTimeStr);

    if (doEventSpec) {
      sprintf( title, "%s Event Spectrum %s:  Peaks %.3lf,%.3lf MHz", 
	       label, dateTimeStr, es[iAFreq], es[iBFreq]); 
      sprintf( plotName, "%s%sFft",outDir,dateTimeStr);
      quickPlot( plotName, (nPlot < MAXPLOTS), "", title, 
		 "Frequency (MHz)", "Intensity",
		 nEChan, es, as, "FFT H", bs, "FFT V", NULL, NULL, NULL, NULL);
      nPlot++;
    }

    if (doEventTime) {
      sprintf( title, "%s Event Time Series %s: %ld Samples", 
	       label, dateTimeStr, tEnd-tStart);
  
      sprintf( plotName, "%s%sEvent",outDir,dateTimeStr);
      quickPlot( plotName, (nPlot < MAXPLOTS), "", title, 
		   "Event Samples", "Voltage",
		 nEvent, &ts[eStart], eas, "H", ebs, "V", 
		 NULL, NULL, NULL, NULL);
      nPlot++;
    }

  } /* end for all events */

  if (doAveSpec && nSpec > 0) {
    fAPeak = fBPeak = 0;
    for (i = 0; i < nChan; i++)          /* normalize the spectra */
      aveASpec[i] = aveASpec[i]/nSpec;
    for (i = 0; i < nChan; i++)          /* normalize the spectra */
      aveBSpec[i] = aveBSpec[i]/nSpec;

    for (i = 0; i < nEChan; i++) 
      aveEventASpec[i] = aveEventASpec[i]/nEventSpec;
    for (i = 0; i < nEChan; i++) 
      aveEventBSpec[i] = aveEventBSpec[i]/nEventSpec;

    aveASpec[0] = aveASpec[nChan-1] = 0;   /* zero end channels */
    aveBSpec[0] = aveBSpec[nChan-1] = 0;   /* zero end channels */
    for (i = 1; i < nChan-1; i++) {
      if (fabs( aveASpec[i]) > fAPeak) {
	iAFreq = i;
	fAPeak = fabs( aveASpec[i]);
      }
      if (fabs( aveBSpec[i]) > fBPeak) {
	iBFreq = i;
	fBPeak = fabs( aveBSpec[i]);
      }
    } /* end for all peak frequency channels */

    aveEventASpec[0] = aveEventASpec[nEChan-1] = 0;
    aveEventBSpec[0] = aveEventBSpec[nEChan-1] = 0;
    fEAPeak = fEBPeak = 0;
    for (i = 1; i < nEChan-1; i++) {
      if (fabs( aveEventASpec[i]) > fEAPeak) {
	iEAFreq = i;
	fEAPeak = fabs( aveEventASpec[i]);
      }
      if (fabs( aveEventBSpec[i]) > fEBPeak) {
	iEBFreq = i;
	fEBPeak = fabs( aveEventBSpec[i]);
      }
    } /* end for all peak frequency channels */

    if (nLongSpec > 0) { 
      for (i = 0; i < nChan; i++) 
	aveLongASpec[i] = aveLongASpec[i]/nLongSpec;
      for (i = 0; i < nChan; i++) 
	aveLongBSpec[i] = aveLongBSpec[i]/nLongSpec;
      
      aveLongASpec[0] = aveLongASpec[nEChan-1] = 0;
      aveLongBSpec[0] = aveLongBSpec[nEChan-1] = 0;
      
      fLAPeak = fLBPeak = 0;
      for (i = 1; i < nChan-1; i++) {
	if (fabs( aveLongASpec[i]) > fLAPeak) {
	  iLAFreq = i;
	  fLAPeak = fabs( aveLongASpec[i]);
	}
	if (fabs( aveLongBSpec[i]) > fLBPeak) {
	  iLBFreq = i;
	  fLBPeak = fabs( aveLongBSpec[i]);
	}
      } /* end for all peak frequency channels */
      sprintf( title, "%s Average Long Event Spectrum: Peaks %.3lf,%.3lf MHz", 
	       label, fs[iLAFreq], fs[iLBFreq]);
      
      sprintf( plotName, "%s%sAveLongSpec",outDir,label);
      quickPlot( plotName, TRUE, "", title, 
		 "Frequency (MHz)", "Intensity",  
		 nChan, fs, aveLongASpec, "Ave A", 
		 aveLongBSpec, "Ave B", NULL, NULL, NULL, NULL);
    } /* if plotting long event Spec */

    sprintf( title, "%s Average Spectrum: Peaks %.3lf,%.3lf MHz", 
	     label, fs[iAFreq], fs[iBFreq]);
      
    sprintf( plotName, "%s%sAveSpec",outDir,label);
    quickPlot( plotName, TRUE, "", title, 
	       "Frequency (MHz)", "Intensity",  
	       nChan, fs, aveASpec, "Ave A", 
	       aveBSpec, "Ave B", NULL, NULL, NULL, NULL);

    nEChan = nEvent/2;
    dFreq = (eFreq - bFreq)/nEChan;  /* calculate frequency interval */
    for (i = 0; i < nEChan; i++) 
      es[i] = bFreq + (i*dFreq);

    sprintf( title, "%s Average Event Spectrum: Peaks %.3lf,%.3lf MHz", 
	     label, es[iEAFreq], es[iEBFreq]);
      
    sprintf( plotName, "%s%sAveEventSpec",outDir,label);
    if (nEventSpec > 0) 
      quickPlot( plotName, TRUE, "", title, 
	       "Frequency (MHz)", "Intensity",  
	       nEChan, es, aveEventASpec, "Ave A", 
	       aveEventBSpec, "Ave B", NULL, NULL, NULL, NULL);
  }

  if (pLog) 
    fprintf( pLog, "# %ld Total Events, %ld short events, %ld long\n",
	     nSpec, nEventSpec, nLongSpec);
  fclose(pLog);

  return(0);
} /* end of eventFilter */

