/* File extractName.c, version 1.2, released 97/05/08 at 10:45:17 retrieved by SCCS
   07/07/18 at 17:59:43

%% extractName() extract the file name part of a full path

:: GBES-C extractName
 
HISTORY:
  970508 GIL return extension correctly
  961107 GIL initial version
 
DESCRIPTION:
extractName() parses a full path name to a file, and extracts the
name part of the file, and the extension to the fileName.  Ie:
if   fullPath == "/s3/ovlbi/out/9611092241.phr.a"
then fileName == "9611092241" and extension == "phr.a"
*/

#include "stdio.h"
#include "string.h"

char *  stripWhite(char *);       /* remove spaces on both ends of a string */

/* internals */

char * extractName(char * fullPath, char * fileName, char * extension)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* extractName() reads in a full file path and name, and returns the fileName*/
/* INPUT:    fullPath   to the input file (NULL terminated)                  */
/* OUTPUTS:  fileName   name part of full path, with extension removed       */
/*           extension  extension of fullPath (all chars after first "."     */
/* returns error message if fileName has zero length, else NULL if OK        */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ int i = 0, j = 0, k = 0, len = strlen( fullPath);

  for (i = 0; i < len; i++) {               /* for all chars in fullPath */
    if (fullPath[i] == '/')                 /* if in start of path */
      j = 0;                                /* restart output name */
    else {
      fileName[j] = fullPath[i];            /* else copy a char */
      j++;                                  /* update output index */
    }
  } /* end for all input characters */
  fileName[j] = '\0';                       /* terminate input string */

  if (j <= 0)
    return("No Name part of File in Path");

  len = strlen( fileName);
  for (j=0; j < len; j++)                   /* for all chars in name */
    if (fileName[j] == '.') {               /* if file has an extension */
      fileName[j] = '\0';                   /* separate file name */
      j++;                                  /* go to next character */
      break;
    }
  k = 0;
  for (i=j; i < len; i ++) {                /* now transfer rest to extension*/
    extension[k] = fileName[i];             /* transfer rest of chars */
    k++;
  } /* end for rest of fileName */
  extension[k] = '\0';                      /* terminate extension */
  return( NULL);
} /* end of extractName() */
