/* File fitsDateTimeStr.c, version 1.1, released  09/03/26 at 09:27:43
   retrieved by SCCS 09/03/26 at 09:27:43

%% Program to write captured events to FITS tables

:: Offline diagnostic

HISTORY:
070821 GIL transform ipcap.c to a subroutine form
070801 P?B initial version in program form.

DESCRIPTION:
eventCaputureIO() reads the latest event from the IBOB
and returns events if found.

DESCRIPTION:
PickIt clone that uses TCP/IP communication to the
iBOB's lwIP low speed ethernet interface.

CHANGES:
* 26 JULY 2007
	- Set initial version at 1.0
	- Runtime measured at 60ms

TODO:
* Add options similar to pickit's
*/

/* Includes used for some general things */
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>  /* Time... is on my side... */
#include <time.h>

/* Includes used for most socket operations */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>

/* Includes used for auxiliary calls: memset, etc. */
#include <string.h>
#include "MATHCNST.H"

/******************************************/
/* This function generates a standardized */
/* date string for use as a filename	  */
/* <YYYY>-<MM>-<DD>T<HH>:<MM>:<SS>.<msms> */
/* Written by Joe Brandt 		  */
/******************************************/
/* externals */
long dayMonthYear2mjd( int day, int month, int year) ;
/* the timeval record may be read from any source */

void fitsDateTimeStr(struct timeval *tv, char *timeString) 
{ struct tm tim;

  gmtime_r(&tv->tv_sec, &tim);

  sprintf( timeString, "%04d-%02d-%02dT%02d:%02d:%02d.%02d", 
	  tim.tm_year+1900, tim.tm_mon+1, tim.tm_mday, 
	  tim.tm_hour, tim.tm_min, tim.tm_sec, (tv->tv_usec*1.e-6);
  
}/* end of fitsDateTimeStr() */

void tvToMjdUtc(struct timeval *tv, long * mjd, double * utc)
{ struct tm tim;

  gmtime_r(&tv->tv_sec, &tim);

  * mjd = dayMonthYear2mjd( tim.tm_mday, tim.tm_mon+1, tim.tm_year);
  * utc = tim.tm_hour + (tim.tm_min/60.) + 
    (tim.tm_sec + (tv->tv_usec*1.e-6)/3600.);  /* convert to hours */
  * utc = * utc * TWOPI/ 24.;
  
}/* end of tv2MjdUtc() */
