/* File fixMidnight.c, version 1.1, released  06/02/24 at 09:55:41 
   retrieved by SCCS 07/07/18 at 17:59:50     

%% GBES utility to fix MJD, UTC calculations of midnight

:: GBES-C offline doppler
 
HISTORY:
  060215 GIL initial version taken from doppler.c

 
DESCRIPTION:
fixMidnight keeps UTC in range 0 to < TWOPI, by making corresponding
adjustments to MJD.
*/
#include <stdio.h>
#include <math.h>
#include "MATHCNST.H"

/* internals */

char * fixMidnight( long * pMjd, double * pUtc)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* fixMidnight() deals with keeping the MJD and utc of times in range        */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ long mjd = *pMjd;
  double utc = *pUtc;

  while ( utc < 0) {              /* first handle before midnight */
    utc += TWOPI;
    mjd --;
  } /* end while before midnight */

  while ( utc >= TWOPI) {         /* next handle after midnight */
    utc -= TWOPI;
    mjd ++;
  } /* end while after midnight */

  *pMjd = mjd;
  *pUtc = utc;
  return (NULL);
} /* end of fixMidnight() */
