/* File gridPlot.c, version 1.3, released  09/04/07 at 20:34:33 
   retrieved by SCCS 09/05/19 at 16:30:30     

%% plot function for plotting grids
:: utility
 
HISTORY:
  090406 GIL allow adding x, y axis labels and two axies different counts
  061025 GIL a 3-d plot of the latest peak data

DESCRIPTION:
Take an Grided set of data produce a 
and plots them. 
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/* internals */

#ifndef NULL
#define NULL  0
#endif

char * gridPlot2( char * plotName, long gv, 
		 char * gnuCmds, char * title, char * object,
		 char * xLabel,  char * yLabel, char * zLabel,
		  long nX, long nY,  double minX, double maxX,
		  double minY, double maxY, double minZ, double maxZ, 
		  char * gridFileName)
/* gridPlot2() takes an X, Y, Z array of data and makes a 3d plot            */
/* by spawing gnuplot and ghostview.   Because the intermediate files are    */
/* deleted at start this program can only reliably be called once per        */
/* execution.  Returns NULL on OK                                            */
/* gridPlot() generates a text file for input to gnuplot and a sequence of   */
/* gnuplot commands.  The gnuplot function is spawned along with ghostview   */
/* to allow viewing of the output plot, with print capability                */
{ char gnuName[256] = "gridPlot2", psName[256] = "gridPlot2", 
    sysStr[256] = "", gifName[256] = "gridPlot2.gif";
  FILE * pGnu = NULL;
 
  if ( plotName != NULL) {
    if (strlen( plotName) > 0) {
      strcpy( gnuName, plotName);
      strcpy( psName, plotName);
      strcpy( gifName, plotName);
    }
  } /* if non-null infile Name() */

  strcat( gnuName, ".gnu");
  strcat( psName, ".ps");
  strcat( gifName, ".gif");

  /* plot postscript file */
  remove(gnuName);
  if ((pGnu = fopen( gnuName, "w")) == NULL) {
    fprintf( stderr, "gridPlot: Error opening gnu file\n");
    return("Error opening gnu file");
  }

  /* remove old file, if present */
  sprintf( sysStr, "gnuplot < %s \n",gnuName);
  system( sysStr);

  /* prepare to write new postscript file */
  remove(gnuName);
  if ((pGnu = fopen( gnuName, "w")) == NULL) {
    fprintf( stderr, "gridPlot: Error opening gnu file\n");
    return("Error opening gnu file");
  }

  fprintf( pGnu, "set terminal postscript color \"Helvetica\" \n");
  fprintf( pGnu, "unset clabel\n");
  fprintf( pGnu, "set output \"%s\"\n", psName);
  fprintf( pGnu, "set xlabel '%s'\n", xLabel);
  fprintf( pGnu, "set ylabel '%s'\n", yLabel);
  fprintf( pGnu, "set zlabel '%s'\n", zLabel);
  fprintf( pGnu, "set title  '%s'\n", title);
  fprintf( pGnu, "set palette maxcolors 100\n");
  fprintf( pGnu, "set contour\n");
  fprintf( pGnu, "set cntrparam levels 10\n");
  fprintf( pGnu, "set view 60,60,1.10\n");

  if (gnuCmds)                 /* if non-NULL cmds pointer */
    if (strlen (gnuCmds) > 0)    /* if any gnuCmds */
      fprintf( pGnu, "%s\n", gnuCmds); /* add to plot to over-ride */

  fprintf( pGnu, "set size 1\n");           /* make ps file fit on page */
  /*  fprintf( pGnu, "splot [%.0lf:%.0lf] [%.0lf:%.0lf] '%s' matrix w l palet title \"%s\"", */
  fprintf( pGnu, "splot [%.0lf:%.0lf] [%.0lf:%.0lf] [%.3lf:%.3lf] '%s' w l palet title \"%s\"", 
	   minX, maxX, minY, maxY, minZ, maxZ, gridFileName, object);
  fprintf( pGnu, "\n");

  fclose( pGnu);
  remove(psName);                           /* now remove old plot file */
  sprintf( sysStr, "gnuplot < %s \n",gnuName);
  system( sysStr);

  if (gv) {                       /* optionally plot for user */
    sprintf( sysStr, "ghostview %s &\n", psName);
    system( sysStr);
  }

  remove(gifName);                           /* now remove old plot file */
  sprintf( sysStr, "echo "" | /opt/local/bin/pstogif -flip cw %s >> /dev/null\n", psName);
  system( sysStr);

  sprintf( sysStr, "chmod 666 %s %s\n", psName, gifName);
  system( sysStr);

  return(NULL);
} /* end of gridPlot2() */

char * gridPlot( char * plotName, long gv, 
		 char * gnuCmds, char * title, char * object,
		 char * xLabel,  char * yLabel, char * zLabel,
		 long n,  char * gridFileName)
/* gridPlot() takes up to 4 data arrays and plots X vs up to 4 Ys */
/* by spawing gnuplot and ghostview.   Because the intermediate files are    */
/* deleted at start this program can only reliably be called once per        */
/* execution.  Returns NULL on OK                                            */
/* gridPlot() generates a text file for input to gnuplot and a sequence of  */
/* gnuplot commands.  The gnuplot function is spawned along with ghostview   */
/* to allow viewing of the output plot, with print capability                */
{ char gnuName[256] = "gridPlot", psName[256] = "gridPlot", sysStr[256] = "", 
    gifName[256] = "gridPlot.gif";
  double minX = 0, maxX = 0, minY = 0, maxY = 0; 
  FILE * pGnu = NULL;
 
  if ( plotName != NULL) {
    if (strlen( plotName) > 0) {
      strcpy( gnuName, plotName);
      strcpy( psName, plotName);
      strcpy( gifName, plotName);
    }
  } /* if non-null infile Name() */

  strcat( gnuName, ".gnu");
  strcat( psName, ".ps");
  strcat( gifName, ".gif");

  minX = minY = 1;
  maxX = maxY = n;
  
  /* plot postscript file */
  remove(gnuName);
  if ((pGnu = fopen( gnuName, "w")) == NULL) {
    fprintf( stderr, "gridPlot: Error opening gnu file\n");
    return("Error opening gnu file");
  }

  /* remove old file, if present */
  sprintf( sysStr, "gnuplot < %s \n",gnuName);
  system( sysStr);

  /* prepare to write new postscript file */
  remove(gnuName);
  if ((pGnu = fopen( gnuName, "w")) == NULL) {
    fprintf( stderr, "gridPlot: Error opening gnu file\n");
    return("Error opening gnu file");
  }

  fprintf( pGnu, "set terminal postscript color \"Helvetica\" \n");
  fprintf( pGnu, "unset clabel\n");
  fprintf( pGnu, "set output \"%s\"\n", psName);
  fprintf( pGnu, "set xrange [%.0lf:%.0lf]\n", minX, maxX);
  fprintf( pGnu, "set yrange [%.0lf:%.0lf]\n", minY, maxY);
  fprintf( pGnu, "set xlabel '%s'\n", xLabel);
  fprintf( pGnu, "set ylabel '%s'\n", yLabel);
  fprintf( pGnu, "set zlabel '%s'\n", zLabel);
  fprintf( pGnu, "set title  '%s'\n", title);
  fprintf( pGnu, "set palette maxcolors 100\n");
  fprintf( pGnu, "set contour\n");
  fprintf( pGnu, "set cntrparam levels 10\n");
  fprintf( pGnu, "set view 60,30,1.25\n");

  if (gnuCmds)                 /* if non-NULL cmds pointer */
    if (strlen (gnuCmds) > 0)    /* if any gnuCmds */
      fprintf( pGnu, "%s\n", gnuCmds); /* add to plot to over-ride */

  fprintf( pGnu, "set size 1\n");           /* make ps file fit on page */
  fprintf( pGnu, "splot '%s' matrix w l palet title \"%s\"", 
	   gridFileName, object);
  fprintf( pGnu, "\n");

  fclose( pGnu);
  remove(psName);                           /* now remove old plot file */
  sprintf( sysStr, "gnuplot < %s \n",gnuName);
  system( sysStr);

  if (gv) {                       /* optionally plot for user */
    sprintf( sysStr, "ghostview %s &\n", psName);
    system( sysStr);
  }

  remove(gifName);                           /* now remove old plot file */
  sprintf( sysStr, "echo "" | /opt/local/bin/pstogif -flip cw %s >> /dev/null\n", psName);
  system( sysStr);

  sprintf( sysStr, "chmod 666 %s %s\n", psName, gifName);
  system( sysStr);

  return(NULL);
} /* end of gridPlot() */
