/* File openOutRevision.c, version 1.4, released  97/09/11 at 14:56:35 
   retrieved by SCCS 07/07/18 at 17:59:42     

%% GBES offline program to read srt file and compile station cmd file

:: GBES-C offline doppler
 
HISTORY:
  970911 GIL - start from last version and work towards first version
  961021 GIL - first argument is a pointer, not pointer pointer
  961008 AHM - modify so opened file name is returned
  961007 AHM - initial version
 
DESCRIPTION:
function which opens the output file.  This function hunts for the
latest revision of the file which is not present.
*/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "MATHCNST.H"
#include "STDDEFS.H"

/* externals */

/* internals */
#ifndef LEN_FTXT
#define LEN_FTXT 256
#endif
#define MAXREVISION 26

int checkOutFile( char * fileName)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* checkOutFile() returns true if the file exists.                           */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ FILE * fp = NULL;

  /* try open for read, if OK then file exists notify */
  if ((fp = fopen(fileName,"r")) != NULL) {
    fclose(fp);
    return(TRUE);
  }
  return(FALSE);   /* else false, file does not exist */
} /* end of checkOutFile() */

/* this function determines the correct version extension (*.a *.b ...) of
   the output file to open and opens it.  the name of the output file and
   the pointer to the file are returned to the calling function */
char * openOutRevision(char * outName, FILE ** pFileName) {
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* IN/OUT  outName      the output file name is modified to be the latest    */
/*                      revision name, if the file already exists            */
/* OUT     fpOut        pointer to the opened output file.                   */
/* Return NULL on success, else error message.                               */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
  FILE * pFile = NULL;
  char fileName[LEN_FTXT]="", alpha[] = "abcdefghijklmnopqrstuvwxyz";
  int length=0, fileExists = FALSE, revisionNumber = MAXREVISION,
    newRevision = MAXREVISION;

  strcpy(fileName, outName);
  length = strlen(fileName);
  fileName[length+2] = '\0';                /* add termination after revision*/
  fileName[length] = '.';                   /* add revision letter separator */

  do {                                      /* for all output revisions */

    fileName[length+1] = alpha[revisionNumber-1];     /* next letter */
    fileExists =  checkOutFile( fileName);  /* does revision exist */
   
    if (!fileExists)                        /* if no file */
      newRevision = revisionNumber;         /* this could be next revision */
    else 
      break;                                /* file exists, newRevision known*/

    revisionNumber --;                      /* prepare to try next revisions */
  } while (revisionNumber > 0);                  /* while more revisions */

  if (newRevision >= MAXREVISION)                /* if max revision already */
    return("unable to create new file name");

  if (!fileExists) {                             /* if no revisions */
    strcpy( fileName, outName);              
    fileExists = checkOutFile( fileName);        /* input without revision */
    if (fileExists)                              /* if input name exists */
      newRevision = 1;                           /* then output is first rev.*/
  } /* end if no file revisions */

  if (fileExists) {
    fileName[length+2] = '\0';              /* add termination after revision*/
    fileName[length] = '.';                 /* add revison letter separator */
    fileName[length+1] = alpha[newRevision-1];   /* next letter */
  }

  /* found a good name - now return file pointer and copy name to output */
  if ((pFile = fopen(fileName,"w")) == NULL)
    return("ERROR...COULD NOT CREATE REVISION");

  *pFileName=pFile;
  strcpy( outName, fileName); 
  return(NULL);
} /* end of openOutRevision() */

