/* File pat2Off.c, version 1.28, released  08/12/15 at 12:00:06 
   retrieved by SC 09/03/26 at 14:49:37     

%% Pointing Error calculation subroutines

:: Offline peakUp pointing offsets

HISTORY:
 080904 GIL make one table for each band in the file
 080903 GIL improve table of fits
 080428 GIL re-calculate offset
 071127 GIL no longer calculate offsets,  find minimum
 071118 GIL check for min == max == constant data, plot dHa,dDec
 071117 GIL new GB43M version
 070328 GIL add source to label to tSys plot
 070328 GIL add source to label to tSys plot
 070327 GIL print out intensities in counts also
 070316 GIL update the zero levels
 070308 GIL include zero level subtraction in tsys estimate
 070306 GIL figure beam width and adjust median size
 070303 GIL add write fit flag
 070208 GIL add option to not print plots
 070207 GIL write out the fit values
 061128 GIL smooth data before fitting
 061027 GIL fix gridding for case of not near source
 061025 GIL finish output of gridded points
 061013 GIL add grid-output and out file name
 061013 GIL use readNextFITS() to read more patterns and getTableCardValue()
 061012 GIL use read frequencies from the TABLE header 
 061011 GIL use plotLabels
 060901 GIL compute gridplot, calc Tsys in Kelvins
 060814 GIL show info to estimate Tsys
 060811 GIL add dra, ddec, intenity output
 060405 GIL deal with more columns in data array
 060321 GIL deal with more columns in data array
 051130 GIL use new quickPlot format
 051129 GIL reorganize the web page
 051115 GIL fix model fitting
 051115 GIL create a web page 
 051110 GIL use quickPlot
 051025 GIL add some plotting diagnostics
 050907 GIL 43m version that works with HA Dec
 971231 GIL keep azimuth value in range 0 to 360 degrees
 970105 GIL use updated CASA position
 960105 GIL use updated CASA position
 950912 GIL read FITS format patterns.
 950127 GIL add comments and put labels in output files
 940805 GIL update to use new 4 channel pattern format
 940412 GIL Initial version

DESCRIPTION:
This function reads in a pattern file and outputs a series of measurements
of the azimuth and elevation offsets as a function of azimuth and elevation
*/

#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "MATHCNST.H"
#include "PEAKUP.H"
#include "POINT.H"
#include "GBESCOORD.H"
#include "STDDEFS.H"
#include "KEYWORD.H"

/* externals */

extern char * angleLimit( double * inOutAngle);
extern char * angleRange( double minAngle, double maxAngle, double * angle);
extern double gastm( long);
extern void obsposn (long, double, double, double, double *, double *); 
extern char * gridPlot( char * plotName, long gv, 
		 char * gnuCmds, char * title, char * object,
			char * xLabel,  char * yLabel, char * zLabel,
		 long n,  char * gridFileName);
extern char * getTableCardValue( char * fitsKeyword, 
		     long * returnType, long * returnLong, 
				 double * returnDouble, char * returnStr);
extern char *stripWhite(char *);  /* remove leading and trailing blanks */
extern char *str2rad ( char *, double *);/* read double from string */
extern char * extractName(char * fullPath, char * fileName, char * extension);
                                    /* Fit measurements to model */
extern struct fit getRms( int band, struct pattern *);
extern void fitPattern( int band, struct pattern *, struct fit *);
extern void fitcpy( struct fit *, struct fit *);
extern char * fourPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n1, double * x1s, double * y1s, char * y1Label,
		  long n2, double * x2s, double * y2s, char * y2Label,
		  long n3, double * x3s, double * y3s, char * y3Label,
		 long n4, double * x4s, double * y4s, char * y4Label);
extern char * quickPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n,  double * xs, 
		  double * y1s, char * y1Label,
		  double * y2s, char * y2Label,
                  double * y3s, char * y3Label,
		  double * y4s, char * y4Label);
extern void patAzElOffset( struct pattern * pPat);
extern char * cvrtuc( char *);
extern void modelBeam( int band, struct fit fit, struct pattern * ppat, 
		       struct pattern * modelPat);
extern char * mjd2str( long, char *);
extern char * rad2str( double, char *, char *);
extern void medianFilter ( int count, int medWid2, double inarr[], 
			   double outarr[]);
extern double median4( double a, double b, double c, double d);
extern double now2Utc();
extern long today2mjd();
extern char * ottFlux( char source[], double epochMjd, 
		long nFreqs, double frequencyMHzs[], 
		double fluxJys[], double errorJys[]);
extern char * initFreqLabels( long nLabels, char * freqLabels[]);
extern char * readLabelFile( long nLabels, char * freqLabels[]);
extern long readNextFITS( long * nPerScan, struct pattern * pPat);
extern long readRawFITS( char * FITSDir, char * FITSFileName, long * nPerScan,
		  struct pattern * pPat);
extern char * setOttSourceName( char * source);
extern char * setOttSourceFluxes( long n, double values[]);
extern double medianSelect(unsigned long k, unsigned long n, double arr[]);
char * getHeaderCardValue( char * fitsKeyword, 
		     long * returnType, long * returnLong, 
			   double * returnDouble, char * returnStr);

/* internals */
struct peakPoint peakPoint;
double lastRefraction = 0.;
long haDec = TRUE;                /* signal haDec computations */
long doWriteFit = FALSE;
#define NFITS       4
double frequencies[8] = {612.,612.,910.,910., 612.,612.,910.,910.},
  bandWidths[8] = {4., 4., 4., 4., 4., 4., 4., 4.};

void wrtPattern( char * patternFile, char * label, struct pattern * ppat);
void writePeak( char * peakFileName, struct fit * ppeak, double peaks[4]);
void subBeam( int, struct fit, struct pattern * ppat, struct pattern * spat);
char * writeOffs( char * offFileName, struct fit * poff, double azOffs[], 
		double elOffs[], char * sourceName);
void fitGaussLin( int, struct fit *, struct pattern *, int);
void fitGaussNonLin( int, struct fit * infit, int ia[], struct pattern * ppat);

#define VERSIONSTR "13.0"
#define WIDTH 4                   
#define MAXSCANS   100
#define SMALLANGLE DEGREE
#define EPSILON    ARCSEC
#define CASARA       (23.391936*15.*DEGREE)
#define CASADEC      (58.837008*DEGREE)
#ifndef HA_DEC 
#define HA_DEC 0
#endif

#define KELVINPERJY 0.295    /* conversion between Tsys K/source JY for 43m */

#define NGRIDX 240
#define NGRIDY 240
double map[NGRIDX][NGRIDY], weight[NGRIDX][NGRIDY];
long nConvolve = 3;          /* convolving kernal half size in cells */
#define NPOWER  8            /* number of bbc intensities */

#ifndef GBES                 /* if the offline system, define functions */
#include "STDDEFS.H"
void taskDelay( int delay)   /* null taskDelay function */
{ return;
}

/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
char * writeAFit( char * fitFileName, struct fit * pfit, long band,
		  char * label, char * sourceName, 
		  char * pols[], double fluxJys[], double tSysCounts[],
		  double * tSysK)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* writeAFit() appends an ascii text string describing the pointing fit to   */
/* the output file.  If the file is not open, writeFit() opens it and adds   */
/* a label string                                                            */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ static FILE *afitfile;
  static int fileOpen = FALSE;
  char fitmsg[1024] = "";
  double tSys = 0.;

  if (fileOpen == FALSE) {           /* if file not open, open it */
    if ((afitfile = fopen( fitFileName,"a+")) == NULL) {
      printf ("writeFit: File not write-able \"%s\"", fitFileName);
      return("Can not Write Output file");
    }
    else {
      fileOpen = TRUE;
    }
    sprintf(fitmsg,
    "# HA  Declination  N    Freq   BW  DeltaX DeltaY Intensity\
 Bmaj   Flux    Tsys  Counts %s\n", label);
  fprintf(afitfile,"%s",fitmsg);
  } /* end if time to open file */
                                         /* record offset from position */
  sprintf(fitmsg,    "%8.5lf %8.5lf %2ld %7.1lf %5.1lf",
	  pfit->max.az, pfit->max.el, band, frequencies[band],
	  bandWidths[band]);
  printf("%s",fitmsg);
  fprintf(afitfile,"%s",fitmsg);

  if (pfit->max.fl[band] > 0.0)
    tSys = KELVINPERJY * fluxJys[band]/(pfit->max.fl[band]);

  sprintf(fitmsg,
	  "%6.1lf %6.1lf %8.3lf %6.1lf %7.1lf %6.1lf %7.0lf \"%sP %s\"\n",
	  pfit->max.daz/ARCMIN, pfit->max.del/ARCMIN, 
	  pfit->max.fl[band], pfit->bmaj/ARCMIN, fluxJys[band],
	  tSys, tSysCounts[band]*3276.8,  pols[band], sourceName);
  printf("%s",fitmsg);
  fprintf(afitfile,"%s",fitmsg);
  *tSysK = tSys;
  return(NULL);
} /* end of writeAFit() */

void logerror( int errorNo, char * msg)
{ printf("%s\n",msg);
}
#endif

/* computed antenna efficiency of 43m as a function of frequency
Table is from Mike Shields email in 2006 April.

Freq (MHz)   Gain (dBi)    Efficiency
  150            34.77        0.6565
  350            41.20        0.5349
  550            44.66        0.4758
  750            48.10        0.5659
  950            50.16        0.5659
 1150            51.01        0.4702
 1350            53.32        0.5804
 1550            54.39        0.5631
 1750            55.04        0.5127
*/

char * parseArguments( int argc, char * argv[], long * outputMode, 
		       long * writeAll, 
		       long * band, long * iPattern, long * doWeb, 
		       char * label, char * title, long * inNameIndex, 
                       char * outputName, char * inputName, char * groupName,
		       double freqMHzs[])
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* parseArguments() takes the input command line arguments and steps through */
/* them parsing pairs of "-? <value>" values.                                */
/* INPUTS argc     count of command line arguments                           */
/*        argv[]   pointers to command line argument strings                 */
/* OUTPUT argWrite pointer to counter for output file name in argv[]         */
/*        argLog  pointer to counter to first input file name in argv[]      */
/* example: pat2Off -begin 0.0 -end 53000. -l 'FE X' -f p log 960220.log */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ long i = 1;
  char * errMsg = NULL, argument[1000] = "", freqStr[1000] = "";

  if (argc < 2) {                       /* give help if no arguments */
    fprintf( stderr,
	     "usage: pat2Off [-A] [-n] [-m <mode>] [-o <outName>] [-band <n>] <patternFile>\n");
    fprintf( stderr,
	     "where <patternFile> is a FITS PeakUp pattern\n");
    fprintf( stderr, 
	     " outputs: pat2Off.<outName> fits to 1 set of measurements\n");
    fprintf( stderr, 
	     "          offSets.<date>    coordinate offsets for all bands\n");
    fprintf( stderr,
	     "          peaks.<date>      file of peaks for all bands\n");
    fprintf( stderr,
	     " where [-A]                 optionally print all fits a file\n");
    fprintf( stderr,
	     " where [-b N]               optional Band number, range 1 to N\n");
    fprintf( stderr,
	     " where [-W]                 optionally write fits to a tSys file\n");
    fprintf( stderr,
	     " where [-M]                 output fit in Az/El \n");
    fprintf( stderr,
	     " where [-g <groupName>]     optional group name string \n");
    fprintf( stderr,
	     " where [-l <labelString>]   optional label string\n");
    fprintf( stderr,
	     " Version %s \n", VERSIONSTR);
    return("No Arguments");
  }

  *inNameIndex = argc-1;

  do {
    if (*argv[i] != '-') {           /* if not a flag, then the file name */
      *inNameIndex = i;
      break;
    }
    strcpy( argument, argv[i]);
    cvrtuc( argument);
    if (strncmp( argument, "-M", 2) == 0) {
      i++;
      if (i >= argc) {
        errMsg = "No value for mode argument";
        break;
      }
      if (*argv[i] == 'a' || *argv[i] == 'A')   /* if Coordinate Mode=az,el*/
	*outputMode = AZEL;                     /* MODE */
    } /* end of if the date argument */
    else if (strncmp(argument, "-O", 2) == 0) { /* if -output fileName */
      i++;
      if (i >= argc) {
        errMsg = "No value for outputName argument";
        break;
      }
      strcpy( outputName, argv[i]);
    }
    else if (strncmp(argument, "-P", 2) == 0) { /* if -pattern # */
      i++;
      if (i >= argc) {
        errMsg = "No value for pattern number argument";
        break;
      }
      sscanf( argv[i], "%ld", iPattern);
    }
    else if (strncmp(argument, "-N", 2) == 0) { /* if -No web data */
      *doWeb = FALSE;
    }
    else if (strncmp(argument, "-L", 2) == 0) { /* if -output fileName */
      i++;
      if (i >= argc) {
        errMsg = "No value for label argument";
        break;
      }
      strcpy( label, argv[i]);
    }
    else if (strncmp(argument, "-F", 2) == 0) { /* if -frequencies */
      i++;
      if (i >= argc) {
        errMsg = "No values for frequencies argument";
        break;
      }
      strcpy( freqStr, argv[i]);
    }
    else if (strncmp(argument, "-W", 2) == 0) { /* if -frequencies */
      doWriteFit = TRUE;
    }
    else if (strncmp(argument, "-T", 2) == 0) { /* if -output fileName */
      i++;
      if (i >= argc) {
        errMsg = "No value for title argument";
        break;
      }
      strcpy( title, argv[i]);
    }
    else if (strncmp(argument, "-G", 2) == 0) { /* if -groupName */
      i++;
      if (i >= argc) {
        errMsg = "No value for group argument";
        break;
      }
      strcpy( groupName, argv[i]);
    }
    else if (strncmp(argument, "-A", 2) == 0) { /* if -groupName */
      * writeAll = TRUE;
    }
    else if (strncmp(argument, "-B", 2) == 0) { /* if -band # */
      i++;
      if (i >= argc) {
        errMsg = "No value for band argument";
        break;
      }
      sscanf( argv[i], "%ld", band);
    }
    i++;
  } while (i < argc && errMsg == NULL);          /* while more arguments */

  if (errMsg) {                                  /* if input error */
    printf("Input Error: %s\n", errMsg);         /* report and quit */
    return("InputError");
  }

  strcpy( inputName, argv[*inNameIndex]);

  return(errMsg);
} /* end of parseArguments */

char * writeHtmlFile( char * groupName, char * plotName, 
		      char * labelA, char * labelB, char * timeStr,
		      long allPlots) 
{ char systemStr[1000] = "", htmlFileName[1000] = "";
  FILE * pF = NULL;

  strcpy( htmlFileName, groupName);
  strcat( htmlFileName, "_");
  strcat( htmlFileName, plotName);
  strcat( htmlFileName, ".htm");
  if ((pF = fopen( htmlFileName,"w+")) == NULL) {  /* open output file */
    printf ("writeHtmlFile: HTML file not write-able \"%s\"", htmlFileName);
    return("Error writing the html file");
  }

  fprintf( pF, "<html><head>\n");

  /* create file header, indicating file should be refreshed */
  fprintf( pF, "<meta http-equiv=\"refresh\" content=\"60\">\n");
  fprintf( pF, 
    "<title>43m Intensity Hour-Log Display</title>\n");
  fprintf( pF, 
	   "<body text=\"#000000\" bgcolor=\"#ffffff\" ");
  fprintf( pF, 
	   "link=\"#880066\" vlink=\"#0000cc\"\n");
  fprintf( pF, "</head><body>\n");
  fprintf( pF, "<CENTER><FONT FACE=helvetica SIZE=\"+2\">\n");
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/\">\n");
  fprintf( pF, "Latest Peak data: %s </a>", timeStr);
  fprintf( pF, "</font></CENTER>\n");
  fprintf( pF, "<center> "); 
  fprintf( pF, "<a href=\"%s_ABCD.htm\"> %s </a> : 3D : ", 
	   groupName, labelA); 
  fprintf( pF, "<a href=\"%s3D_A.htm\"> A </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_B.htm\"> B </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_C.htm\"> C </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_D.htm\"> D </a> - \n", groupName); 

  fprintf( pF, "<a href=\"%s_2D_ABCD.htm\"> 2D</a>: ", groupName); 
  fprintf( pF, "<a href=\"%s_A.htm\"> A </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_B.htm\"> B </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_C.htm\"> C </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_D.htm\"> D </a>\n", groupName); 
  fprintf( pF, "</center><center>\n"); 
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/mlln/operate\"> ");
  fprintf( pF, "43m Status</a> - \n");
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/mlln/bbc/\"> ");
  fprintf( pF, "Current Intensities</a> - \n");
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/\">");
  fprintf( pF, "NRAO</a> \n");
  fprintf( pF, "</center>\n");
  fprintf( pF, "<center>\n");
  if (allPlots) {
    fprintf( pF, "<table><tr>\n");
    fprintf( pF, "<table><tr>\n");
    fprintf( pF, " <td><a href=\"%s3D_A.htm\">", groupName);
    fprintf( pF, "<img src=\"%s3D_A.gif\" width=300></a></td>\n", groupName);
    fprintf( pF, " <td><a href=\"%s3D_B.htm\">", groupName);
    fprintf( pF, "<img src=\"%s3D_B.gif\" width=300></a></td>\n", groupName);
    fprintf( pF, "</tr><tr>\n");
    fprintf( pF, " <td><a href=\"%s3D_C.htm\">", groupName);
    fprintf( pF, "<img src=\"%s3D_C.gif\" width=300></a></td>\n", groupName);
    fprintf( pF, " <td><a href=\"%s3D_D.htm\">", groupName);
    fprintf( pF, "<img src=\"%s3D_D.gif\" width=300></a></td>", groupName);
    fprintf( pF, "</tr>\n</table>\n");
  }
  else {
    fprintf( pF, "<a href=\"%s_%s.ps\">", groupName, plotName);
    fprintf( pF, "<img src=\"%s_%s.png\"</a>", groupName, plotName);
  }
  fprintf( pF, "</center>\n");
  fprintf( pF, "<center> "); 
  fclose( pF); 

  sprintf( systemStr, "chmod 666 %s", htmlFileName);
  system(systemStr);  

  return(NULL);
} /* end of writeHtmlFile */

#define PEAKFREQMHZ  912.5        /* 43m beam width measurement frequency */
#define PEAKWIDTH     34.5        /* 43m beam width in Arcminutes */

char * getFreqBandPol( int band, double * freq, double * bandWidth, 
		       char * polarization)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* getFreqBandPol() assumes an open Pattern FITS file is present and returns */
/* recorded values from the fits file.  If the values are not present */
/* the input values are not modified.  Frequencies are in Hz            */
/* null on no errors reading file */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char tempStr[100] = "", keyword[100] = "", * errMsg = NULL;
  long longValue = 0, type = 0; 
  double doubleValue = 0; 

  sprintf( keyword,"T4FREQ%02d", band+1);
  errMsg = getTableCardValue( keyword, &type, &longValue,
			      &doubleValue, tempStr);
  if (errMsg == NULL) {
    if (type == FITSFLOAT) {
      *freq = doubleValue;
    }
  } /* end if could read frequency */

  sprintf( keyword,"T4BAND%02d", band+1);
  errMsg = getTableCardValue( keyword, &type, &longValue,
			      &doubleValue, tempStr);
  if (errMsg == NULL) {
    if (type == FITSFLOAT) {
      *bandWidth = doubleValue;
    }
  } /* end if could read frequency */

  sprintf( keyword,"T4POLA%02d", band+1);
  errMsg = getTableCardValue( keyword, &type, &longValue,
			      &doubleValue, tempStr);
  if (errMsg == NULL) {
    if (type == FITSSTRING) {
      strcpy( polarization, tempStr);
    }
  } /* end if could read frequency */

  fprintf( stderr, "Frequency: %10.3lf, bandwidth = %8.3lf MHz, Stokes %s\n",
	   *freq*1.E-6, *bandWidth*1.E-6, polarization);

  return(errMsg);
} /* end of getFreqBandPolarization() */

char * gridPattern( double ha0, double dec0, struct pattern * pPat, long band) 
{ char * errMsg = NULL;
  long i = 0, j = 0, k = 0, iX = 0, jY = 0, n = 0, i0 = NGRIDX/2, 
    j0 = NGRIDY/2, nX = 0, nY = 0, nIn = 0;
  double dR2 = 0., x = 0, y = 0, dX = 0, dY = 0, w = 0., 
    beamFactor = 9.*ARCMIN*ARCMIN, dX0 = 1.0*ARCMIN, dY0 = 1.0*ARCMIN;

  if (pPat == NULL)
    return( "Null Input pattern");
  if (pPat->n < 1)
    return( "No points in input pattern");
  else
    n = pPat->n;
  
  for (i = 0; i < NGRIDX; i++) {          /* init map and weighs */
    for (j = 0; j < NGRIDY; j++) {
      map[i][j] = weight[i][j] = 0.;
    }
  }
    
  for (k = 0; k < n; k++) {            /* for all points in pattern */
    x = pPat->d[k].daz - ha0;
    y = pPat->d[k].del - dec0;
    iX = x/dX0 + i0;
    jY = y/dY0 + j0;
    if (k%200 == 0)
      fprintf( stderr, "Point %4ld: %7.1lf,%7.1lf => %4ld,%4ld\n",
	       k, x/ARCMIN, y/ARCMIN, iX, jY);
    for (i = iX-nConvolve; i < iX+nConvolve+1; i++) {
      if (i >= 0 && i < NGRIDX) {
	dX = (i - iX)*dX0;
	nX++;
	for (j = jY-nConvolve; j < jY+nConvolve+1; j++) {
	  if (j >= 0 && j < NGRIDY) {
	    nY++;
	    dY = (j - jY)*dY0;
	    dR2 = (dX*dX) + (dY*dY);
	    w = exp( - beamFactor/dR2);
	    map[i][j] = map[i][j] + (pPat->d[k].fl[band] * w);
	    weight[i][j] = weight[i][j] + w;
	    nIn ++;
	  } /* end if j on grid */
	} /* end for all j in convolving function */
      } /* end if i on grid */
    } /* end for all i in convolving function */
  } /* end for all points in pattern */

  for (i = 0; i < NGRIDX; i++) { /* now normalize the grid */
    for (j = 0; j < NGRIDY; j++) {
      if (weight[i][j] > 0.) 
	map[i][j] = map[i][j]/weight[i][j];
    } /* end for all j */
  } /* end for all i */

  fprintf( stderr, "gridPattern: Convolved %ld out of %ld points (%ld, %ld)\n",
	   nIn, n, nX, nY);
  return(errMsg);
} /* end of gridPattern () */

char * writeHtml3D( char * groupName, char * plotName, 
		      char * labelA, char * labelB, char * timeStr) 
{ char systemStr[1000] = "", htmlFileName[1000] = "";
  FILE * pF = NULL;

  strcpy( htmlFileName, groupName);
  strcat( htmlFileName, "3D_");
  strcat( htmlFileName, plotName);
  strcat( htmlFileName, ".htm");
  if ((pF = fopen( htmlFileName,"w+")) == NULL) {  /* open output file */
    printf ("writeHtml3D: HTML file not write-able \"%s\"", htmlFileName);
    return("Error writing the html file");
  }

  fprintf( pF, "<html><head>\n");

  /* create file header, indicating file should be refreshed */
  fprintf( pF, "<meta http-equiv=\"refresh\" content=\"60\">\n");
  fprintf( pF, 
    "<title>43m Peak</title>\n");
  fprintf( pF, 
	   "<body text=\"#000000\" bgcolor=\"#ffffff\" ");
  fprintf( pF, 
	   "link=\"#880066\" vlink=\"#0000cc\"\n");
  fprintf( pF, "</head><body>\n");
  fprintf( pF, "<CENTER><FONT FACE=helvetica SIZE=\"+2\">\n");
  fprintf( pF, 
 "<a href=\"http://www.gb.nrao.edu/\">\n");
  fprintf( pF, "Peak data: %s </a>", timeStr);
  fprintf( pF, "</font></CENTER>\n");
  fprintf( pF, "<center> "); 
  fprintf( pF, "<a href=\"%s_ABCD.htm\"> %s </a> - : 3D : ", 
	   labelB, labelA); 
  fprintf( pF, "<a href=\"%s3D_A.htm\"> A </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_B.htm\"> B </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_C.htm\"> C </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s3D_D.htm\"> D </a> - \n", groupName); 
  fprintf( pF, "<a href=\"%s_2D_ABCD.htm\"> 2D </a> : ", groupName); 
  fprintf( pF, "<a href=\"%s_A.htm\"> A </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_B.htm\"> B </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_C.htm\"> C </a> - ", groupName); 
  fprintf( pF, "<a href=\"%s_D.htm\"> D </a>\n", groupName); 
  fprintf( pF, "</center> "); 
  fprintf( pF, "<center> "); 
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/mlln/operate\"> ");
  fprintf( pF, "43m Status</a> - \n");
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/mlln/bbc/\"> ");
  fprintf( pF, "Current Intensities</a> - \n");
  fprintf( pF, "<a href=\"http://www.gb.nrao.edu/\"> ");
  fprintf( pF, "NRAO</a> \n");
  fprintf( pF, "</center>\n");
  fprintf( pF, "<center>\n");
  fprintf( pF, "<a href=\"%s3D_%s.ps\">", groupName, plotName);
  fprintf( pF, "<img src=\"%s3D_%s.gif\"</a>", groupName, plotName);
  fprintf( pF, "</center>\n");
  fprintf( pF, "<center> "); 
  fclose( pF); 

  sprintf( systemStr, "chmod 666 %s", htmlFileName);
  system(systemStr);  

  return(NULL);
} /* end ofw riteHtml3D */

char * writeGrid( char * fileName)
{ char * errMsg = NULL;
  long i = 0, j = 0;
  FILE * gFile = NULL;

  gFile = fopen ( fileName, "w+");
  if ( gFile == NULL) {
    fprintf( stderr, "Error opening output grid file: %s\n", fileName);
    return( "Error opening grid file");
  }

  for (i = 0; i < NGRIDX; i++) {          /* write all points in grid */
    for (j = 0; j < NGRIDY; j++) {
      fprintf( gFile, "%.3lf ", map[i][j]); 
    } /* end for all j */
    fprintf( gFile, "\n");
  } /* end for all i */
  fclose(gFile);                          /* important! close file or no plot*/

  return(errMsg);
} /* end of writeGrid() */

long checkGrid()
/* checkGrid() checks the grid for non-constant values.  If there is   */
/* checkGrid() returns TRUE if there is data variation, else False */
{ long i = 0, j = 0, n = 0;
  double mean = 1, sum2 = 0, sum = 0, rms = 0, minf = 1e9, maxf = -1e9;

  for (i = 0; i < NGRIDX; i++) {          /* init map and weighs */
    for (j = 0; j < NGRIDY; j++) {
      if (map[i][j] != 0.) {
	sum = sum + map[i][j];
	n++;
	if (minf > map[i][j])
	  minf = map[i][j];
	else if (maxf < map[i][j])
	  maxf = map[i][j];
      }
    }
  } /* end for all i */

  if (n > 0) 
    mean = sum/(double)n;

  for (i = 0; i < NGRIDX; i++) {          /* init map and weighs */
    for (j = 0; j < NGRIDY; j++) {
      if (map[i][j] != 0.) 
	sum2 = sum2 + ((map[i][j]-mean)*(map[i][j]-mean));
    }
  } /* end for all i */

  if (n > 0) {
    rms = sum2/((double)n-1.);
    fprintf( stderr, "checkGrid: %ld non zero Grid points, mean %10.3lf\n",
	     n, mean);
    fprintf( stderr, "Min: %10.3lf,  Max: %10.3lf,  Rms: %10.3lf\n",
	     minf, maxf, rms);
  }
  else {
    fprintf( stderr, "checkGrid: NO non-zero Grid points!\n");
    return(FALSE);
  }
  return(TRUE);
} /* end of checkGrid() */

char * raDecToHaDecNow( long mjd, double utc, double raJ2000, double decJ2000,
			double * haNow, double * decNow)
{ static double lstMidnight = 0;
  static long lastMjd = 0;
  double longitude = +79.835101*DEGREE, lst = 0, raNow = 0;

  /* must recompute lstMidnight for each new day */
  if (mjd != lastMjd) 
    lstMidnight = gastm(mjd) - longitude;
  lastMjd = mjd;
  angleLimit( &lstMidnight);

  lst = lstMidnight + (utc * SID_SOLAR);      
  angleLimit( &lst);          /* keep lst in range 0 to TWO PI */

  /* now convert to current ra, dec */
  obsposn( mjd, utc, raJ2000, decJ2000, &raNow, decNow);

  /* correct for HA convention that west is positive */
  *haNow = lst - raNow;
      
  if (*haNow > PI)
    *haNow = *haNow - TWOPI;
  else if (*haNow < -PI)
    *haNow = TWOPI + *haNow;

  return(NULL);
} /* end of raDecToHaDecNow() */

void writeAllPeaks( char * allFileName, char * sourceName, 
		    struct pattern * pPat, char * pols[],
		    double peaks[], double peakErrs[], 
		    double azOffs[], double elOffs[])
/* output a measured fit to filename */
{ static FILE *peakfile;
  static int fileOpen = FALSE;
  long i = 0;
  char peakmsg[255] = "", dateStr[32] = "", timeStr[32] = "";

  if (fileOpen == FALSE) {
    if ((peakfile = fopen( allFileName,"a+")) == NULL) {
      printf ("writeAllPeak: File not write-able \"%s\"", allFileName);
      return;
    }
    else {
      fileOpen = TRUE;
    }
    sprintf(peakmsg,
    "#  HA  Declination  Freq Peak +/- dHa,Dec, date time source \n");
    fprintf(peakfile,"%s",peakmsg);
  } /* end if time to open file */
                                         /* record peakset from position */
  sprintf(peakmsg, "%8.3lf %8.3lf ",
	  pPat->az/DEGREE, pPat->el/DEGREE);
  fprintf( peakfile, "%s", peakmsg);
  for ( i = 0; i < 4; i++) {
    sprintf( peakmsg, "%s %7.2lf %6.3lf+/-%6.3lf %5.1lf,%5.1lf ",
	     pols[i], frequencies[i], peaks[i], peakErrs[i], 
	     azOffs[i]/ARCMIN, elOffs[i]/ARCMIN);
    fprintf( peakfile, "%s", peakmsg);
  }
  mjd2str( pPat->mjd, dateStr);
  rad2str( pPat->utc, "h.0", timeStr);
  timeStr[strlen(timeStr)-1] = EOS;
  sprintf(peakmsg, " %s %s %s\n", &dateStr[2], timeStr, sourceName);
  fprintf(peakfile,"%s",peakmsg);
} /* end writeAllPeaks() */

int main( int argc, char * argv[])
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* main() = pat2Off() takes command line arguments of:                       */
/* INPUT patternFileName     name of file containing raw pattern measurements*/
/*       dateString          extenstion added to output files                */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char fitName[132] = "", offName[132] = "", peakName[132] = "", 
    sourceName[132] = "", extension[132] = "", inputName[1000] = "",
    outName[1000] = "fit", * errMsg = NULL, groupName[1000] = "", 
    htmlFileName[1000] = "patternABCD.htm", polarization[1000] = "",
    gnuCmds[1000] = 
      "set xtics 100\nset data style linespoints \nset size 1.2,.75",
    allFileName[132] = "allPeakFits.txt",
    gridName[1000] = "gridPattern.txt", ottSource[32] = "",
    labelB[132] = "Fit", label[132] = "", * chars[] = {"A", "B", "C", "D"},
    title[132] = "", dateStr[32] = "", timeStr[32] = "",
    * freqLabels[NPOWER], labelX[132] = "Sample Number", 
      labelY[132] = "Intensity (Counts)",
    * pols[NPOWER];
  long i = 0, j = 0, k = 0, nAve = 0, nXPeak = 0, nKuPeak = 0, iName = 0,
    angleMode = HA_DEC, nPerScan = 0, n = 0, band = 1, startNameIndex = 1,
    doWeb = TRUE, nFreq = 8, todayMjd = today2mjd(), nBytes = 0, n8 = 1,
    nLabels = NPOWER, iPattern = 0, nPattern = 10, medianWidth = 10,
    nTSysV = 0, nTSysH = 0, dataVariation = TRUE, writeAll = FALSE,
    goodData = FALSE;
  double bmin = 0, tSysCounts[NFITS], freq = 0, bandWidth = 0, 
    azOffs[NFITS], elOffs[NFITS], peaks[NFITS], dAz = 0,
    peakErrs[NFITS], haS[MAXTIME], fluxS[NFITS][MAXTIME], decS[MAXTIME], 
    aveAzOff = 0, aveElOff = 0, XPeak = 0, KuPeak = 0, ha0 = 0, dec0 = 0,
    aveKuWidth = 0, aveXWidth = 0, sourceRa = 0, sourceDec = 0,
    medians[MAXTIME], fluxJys[NFITS] = { 0, 0, 0, 0}, datas[MAXTIME],
    errorJys[NFITS] = {0, 0, 0, 0}, beamWidth = PEAKWIDTH, tSysK = 0,
    tSysVs[MAXTIME], tSysHs[MAXTIME], freqVs[MAXTIME], freqHs[MAXTIME],
    sumX = 0, sumY = 0, weightX = 0, weightY = 0, endFluxs[8],
    haNow = 0, decNow = 0;
  struct pattern pattern, subPat, lastPat, modelPat, medianPattern,
    nextPattern;
  struct fit fitRms, fit, subFit, fits[NFITS];

  errMsg = parseArguments( argc, argv, &angleMode, &writeAll, 
			   &band, &nPattern,
			   &doWeb, label, title, &startNameIndex, 
			   outName, inputName, groupName,
			   frequencies);
  if (errMsg)
    return(-1);

  initFreqLabels( nLabels,freqLabels);
  readLabelFile( nLabels, freqLabels);

  strcat( fitName, outName);
  strcat( fitName, "43mPeaks.txt");
  strcpy( offName, "offSets.");
  strcat( offName, outName);
  strcpy( peakName, "peaks.");
  strcat( peakName, outName);

  remove( htmlFileName);

  for (i = 0; i < MAXTIME; i++) {
    freqVs[i] = freqHs[i] = tSysVs[i] = tSysHs[i] = 0;
  }

  if (band < 1 || band > NFITS)
    band = 1;

  if (strlen(label) < 1)
    sprintf( label,"%s", chars[band-1]);

  /* for all input FITS files */
  for (iName = startNameIndex; iName < argc; iName++) {
    strcpy( inputName, argv[iName]);

    /* for all patterns in FITS data file, up to one we want */
    for (iPattern = 0; iPattern < nPattern; iPattern ++) {
      /* read all data in a pattern */
      if (iPattern == 0) 
        nBytes = readRawFITS( "", inputName, &nPerScan, &nextPattern);
      else
        nBytes = readNextFITS( &nPerScan, &nextPattern);

      if (nBytes <= 0) /* if end of file, exit */
        break;

      /* for all input measurements */      
      for (i = 0; i < NPOWER; i++) {
	pols[i] = NULL;
	errMsg = getFreqBandPol( (int)i, &freq, &bandWidth, polarization);
	if (errMsg == NULL) {
	  frequencies[i] = freq*1.E-6;
	  bandWidths[i] = bandWidth*1.E-6;
	  pols[i] = malloc( 100);
	  strcpy( pols[i], polarization);
	  sprintf( freqLabels[i], "%s %8.3lf MHz", 
		   pols[i], frequencies[i]);
	} /* end if can read frequency and bandwidth */
      } /* end for all frequency bands */

      memcpy( (char *)&pattern, (char *)&nextPattern, 
		sizeof( struct pattern));
    } /* end for all patterns */

    /* if source is CASA, reset reference coordinate */
    if ( pattern.ra < CASARA+SMALLANGLE && pattern.ra > CASARA-SMALLANGLE &&
	 pattern.dec< CASADEC+SMALLANGLE && 
	 pattern.dec > CASADEC-SMALLANGLE) {
      pattern.ra = CASARA;                 /* reset reference position */
      pattern.dec = CASADEC;               /* reset reference position */
    } /* end if source is casa */

    sourceRa = pattern.ra;
    sourceDec = pattern.dec;
    extractName( inputName, sourceName, extension);

      for (i = 0; i < strlen(sourceName); i++) 
	if (sourceName[i] == '_')
	  sourceName[i] = EOS;
	if (sourceName[i] == ' ')
	  sourceName[i] = EOS;
      strcpy( ottSource, sourceName);
      ottSource[6] = EOS;

      /*      patAzElOffset( &pattern);   calculate angle offsets */
      /* for all patterns in pattern file */
      for(i = 0; i < 200000 && pattern.n > 0; i++) {
	
	errMsg = ottFlux( ottSource, todayMjd, nFreq, 
			frequencies, fluxJys, errorJys);
	if (errMsg) {
	  fprintf ( stderr, "Error obtaining OTT flux for %s: %s\n",
		    sourceName, errMsg);
	  for (i = 0; i < NFITS; i++)
	    fluxJys[i] = 1000.;
	} /* end if error reading ott Fluxes */
	else
	  fprintf( stderr, "Source: %s  freq=%8.3lf  => flux=%10.2lf\n",
		   sourceName, frequencies[0], fluxJys[0]);
    
      nAve = nXPeak = nKuPeak = 0;         /* init counts for this pattern */
      aveAzOff = aveElOff = XPeak = KuPeak = aveKuWidth = aveXWidth = 0;
      
      n = pattern.n;
      n8 = n/8;                           /* ends of each movement */

      /* keep the azimuth offsets between -PI and PI */
      for( j = 0; j < n; j++) {
        raDecToHaDecNow( pattern.mjd, pattern.d[j].utc, sourceRa, sourceDec,
			 &haNow, &decNow);
	pattern.d[j].daz = pattern.d[j].az - haNow;
	pattern.d[j].del = pattern.d[j].el - decNow; 
	if (pattern.d[j].daz > PI)
	  pattern.d[j].daz = TWOPI - pattern.d[j].daz;
	else if (pattern.d[j].daz < -PI)
	  pattern.d[j].daz = TWOPI + pattern.d[j].daz;
	if (pattern.d[j].daz > PI/2.)
	  pattern.d[j].daz = PI - pattern.d[j].daz;
	else if (pattern.d[j].daz < -PI/2.)
	  pattern.d[j].daz = PI + pattern.d[j].daz;
      }
      dAz = pattern.d[1].daz - pattern.d[n/8].daz;
      if (dAz < 0.)
	dAz = -dAz;
      dAz = dAz/((n/8)-1);
      if (dAz == 0.) {
	fprintf( stderr, "Small Azimuth step between points; dAz = 0\n");
	dAz = 0.25*ARCMIN;
      }
      else 
	fprintf( stderr, "Azimuth step between points; dAz = %7.2lf'\n",
		   dAz/ARCMIN);
	
      for (k = 0; k < NFITS; k++) {       /* for all bands of data */

        if (k == 0)
	  goodData = FALSE;

	for( j = 0; j < n; j++)           /* transfer data to work array */
	  datas[j] = pattern.d[j].fl[k];

        /* remove isolated high points */
	medianFilter( n, 4, datas, medians);

        fprintf( stderr, "Band %ld (%ld): data,median %7.1lf %7.1lf %7.1lf\n",
		 k, n, pattern.d[n/2].fl[k], datas[n/2], medians[n/2]);

	for( j = 0; j < n; j++)  /* from now on, only use filtered data */
          datas[j] = medians[j];

        for( j = 0; j < 8; j ++) {    /* get representative off source counts*/
	  if (((j*n8)+2) < n)
	    endFluxs[j] = medians[(j*n8)+2];
  	  else
	    endFluxs[j] = medians[n-2];
        } /* end for 8 values at ends of scan */

	tSysCounts[k] = medianSelect( 3, 8, endFluxs);

        /* finde appropriate median width size */
	beamWidth = PEAKWIDTH;  /* beam FWHM in arcminutes */
        freq = frequencies[k];
        if (freq > 0.)
	  beamWidth = PEAKWIDTH * PEAKFREQMHZ / freq;        	

        /* compute medianWidth in integer ghannels */
        medianWidth = 6 * beamWidth * ARCMIN / dAz;
	fprintf( stderr, "median width: %ld/%ld for beamWidth %7.2lf/%7.2lf\n",
		 medianWidth, n, beamWidth, dAz/ARCMIN);

        /* compute median value for baseline removal */
	medianFilter( n, medianWidth, datas, medians);

	/* just remove a single nominal tSys value */
	for( j = 0; j < n; j++) 
	  pattern.d[j].fl[k] = datas[j] - tSysCounts[k];

	memcpy( (char *)&medianPattern, (char *)&pattern, 
		sizeof( struct pattern));
	
	/* now compute intensities relative to Tsys */
	if (tSysCounts[k] > 0.001) {  /* if a valid Tsys */
	  for( j = 0;  j < n; j++) {
	    pattern.d[j].fl[k] = pattern.d[j].fl[k]/tSysCounts[k];
	    medianPattern.d[j].fl[k] = medianPattern.d[j].fl[k]/tSysCounts[k];
	  }
	}

	fitRms = getRms( k, &medianPattern);/* get RMS before subtract beam */
        if (fitRms.max.fl[k] < 0.01)       /* if an insignficant peak */
	  continue;

	azOffs[k] = 0;                     /* initialize values */
	elOffs[k] = 0;
	peaks[k] = 0;
	peakErrs[k] = 0;
	sumX = sumY = weightX = weightY = 0;

	if (fitRms.rms > 0.0) {            /* if a valid RMS */
	  if (fitRms.max.fl[k]/fitRms.rms < 3.)  /* if an insignficant peak */
	    continue;
	}
	else                               /* else no data, skip rest */
	  continue;

	if (fitRms.rms <= 0.00001) {       /* if no signal */
	  fitcpy( &fit, &fitRms);            /* keep RMS info */
	  continue;
	} /* end if zero input level */

	fitPattern( k, &medianPattern, &fit); /* fit gaussian to data */   
	bmin =     fit.bmin;               /* record beam shape */
	fit.bmin = fit.bmaj;               /* subtract a circular beam */
	
	subBeam( k, fit, &medianPattern, &subPat);  /* subtract fit from data */
	subFit = getRms( k, &subPat);      /* get rms after subtraction */

	/* keep all fits for latter plotting */
	memcpy( (char *)&fits[k], (char *)&fit, sizeof(fit));

	printf("pat2Off: chan %ld rms before %le, and after %le (bmaj = %le)\n",
	       k, fit.rms, subFit.rms, fit.bmaj/ARCMIN);

	/* keep several sigma fits */
	if ((fit.max.fl[k] > 5.*subFit.rms) && (fit.bmaj > 10.*ARCMIN) &&
	    (fit.bmaj < 240.*ARCMIN) && 
	    (fit.max.daz < 20.*ARCMIN) && (fit.max.daz > -20.*ARCMIN)) {
	  goodData = TRUE;
	  peaks[k] = fit.max.fl[k];        /* record peaks */
 	  peakErrs[k] = subFit.rms;           
	  fit.rms = subFit.rms;           
	  fit.bmin = bmin;
	  nAve++;
	  azOffs[k] = fit.max.daz;
	  elOffs[k] = fit.max.del;
	  if (k < 2) {
	    nXPeak ++;
	    XPeak += fit.max.fl[k];
	    aveXWidth += fit.bmaj;
	    aveAzOff += fit.max.daz;
	    aveElOff += fit.max.del;
	  }
	  else {
	    nKuPeak ++;
	    KuPeak += fit.max.fl[k];
	    aveKuWidth += fit.bmaj;
	  }
	  printf("\n");
	}
	else {
	  printf("fit discarded\n\n");
	  continue; 
	}

        if (frequencies[k] > 0. && doWriteFit) {
	  writeAFit( fitName, &fit, k, inputName, sourceName,
		   pols, fluxJys, tSysCounts,
		     &tSysK); 
          if (tSysK < 900. && tSysK > 30.) {
	    if (strstr( pols[k], "V") != NULL) {
	      tSysVs[nTSysV] = tSysK;
	      freqVs[nTSysV] = frequencies[k];
	      nTSysV++;
	    }
	    else {
	      tSysHs[nTSysH] = tSysK;
	      freqHs[nTSysH] = frequencies[k];
	      nTSysH++;
	    }
	  } /* if a valid fit */
	} /* end if saving the tSys values */
      } /* end of for all bands */

      memcpy( (char *)&lastPat, (char *)&pattern, sizeof( pattern));

      if (nAve > 0) {                      /* if any good data in pattern */
	if (nXPeak > 0) {                  /* if any good X band data */
	  fit.max.daz = aveAzOff / nXPeak; /* transfer for output */
	  fit.max.del = aveElOff / nXPeak;
	  XPeak /= nXPeak;
	  fit.max.fl[0] = XPeak;
	  fit.bmaj = aveXWidth / nXPeak;
	}
	else {
	  fit.max.fl[0] = 0;               /* else zero for output */
	  fit.bmaj = 0;
	}
	if (nKuPeak > 0) {                 /* if any good Ku band Data */
	  KuPeak /= nKuPeak;               /* transfer for output */
	  fit.max.fl[2] = KuPeak;
	  fit.bmin = aveKuWidth / nKuPeak;
	}
	else {                             /* else zero for output */
	  fit.max.fl[2] = 0;
	  fit.bmin = 0;
	}
	fit.bpa = pattern.utc;             /* pass LST time (radians) as bpa */
	/* pointing offsets file */
	/*	writeFit( fitName, &fit, inputName, sourceName);  */
	writeOffs( offName, &fit, azOffs, elOffs, sourceName); 
	writePeak( peakName, &fit, peaks); /* peak calibration file */
      } /* end if any good data in pattern */

      if (iPattern == nPattern && (k == (NFITS-1)))  /* if finished pattern */
	break;                             /* exit loop */
      readRawFITS( "", inputName, &nPerScan, &pattern);
      /* patAzElOffset( &pattern); */
    }  /* end of for all bands */

    if (writeAll) {
      if (goodData)      
	writeAllPeaks( allFileName, sourceName, &lastPat, pols, peaks, 
		       peakErrs, azOffs, elOffs);
      else
	fprintf( stderr, "No Good Fits to this scan: %ld\n", iPattern);
    }
  } /* end for all file names */

  /* get median position of middle half of data for gridding */
  n = lastPat.n;
  for (j = 0; j < n; j++) {
    haS[j] = lastPat.d[j].daz;
    decS[j] = lastPat.d[j].del;
  }
  ha0 = medianSelect( n/4, n/2, &haS[n/4]);
  dec0 = medianSelect( n/4, n/2, &decS[n/4]);

  if (doWeb) {
    errMsg = gridPattern( ha0, dec0, &lastPat, band-1);
    if (errMsg)
      fprintf( stderr, "error gridding pattern: %s\n", errMsg);

    dataVariation = checkGrid();

    writeGrid( gridName);
  }

  /* transfer coordinates for plot */
  for (j = 0; j < n; j++) {
    fluxS[0][j] = lastPat.d[j].fl[0];
    fluxS[1][j] = lastPat.d[j].fl[1];
    fluxS[2][j] = lastPat.d[j].fl[2];
    fluxS[3][j] = lastPat.d[j].fl[3];
  }
  if (dataVariation) {
    modelBeam( band-1, fits[band-1], &lastPat, &modelPat);
  }  
  else {
    for (j = 0; j < n; j++) {
      modelPat.d[j].fl[0] = 0;
      modelPat.d[j].fl[1] = 0;
      modelPat.d[j].fl[2] = 0;
      modelPat.d[j].fl[3] = 0;
    }
  }

  for (j = 0; j < n; j++)         /* just plot in time order */
    haS[j] = j;

  mjd2str( lastPat.mjd, dateStr);
  rad2str( lastPat.utc, "h.0", timeStr);

  if (strlen(groupName) < 1)
    strcpy(groupName,"pattern");
 
  sprintf( outName,"%s_2D_ABCD", groupName);

  if (strlen(title) < 1)
    sprintf( title,"43m Pointing Model (%s %s %s)", 
	     sourceName, &dateStr[2], timeStr);

   if (doWeb) {
     quickPlot(outName, FALSE, gnuCmds, title,
	     labelX, labelY,
	     n,  haS, 
	     &fluxS[0][0], freqLabels[0],
	     &fluxS[1][0], freqLabels[1],
	     &fluxS[2][0], freqLabels[2],
	     &fluxS[3][0], freqLabels[3]);
  
     writeHtmlFile( groupName, "ABCD", sourceName, "", timeStr, TRUE);
     writeHtmlFile( groupName, "2D_ABCD", sourceName, "", timeStr, FALSE);
   } /* end if writting web page plots */

  for (j = 0; j < n; j++) {
    fluxS[0][j] = lastPat.d[j].fl[band-1];
    fluxS[1][j] = modelPat.d[j].fl[band-1];
  }

  /*  n = n/2; do not plot only the last half of the data */

  if (iPattern == 0)
    iPattern = nPattern;

  mjd2str( lastPat.mjd, dateStr);
  rad2str( lastPat.utc, "h.0", timeStr);

  if (strlen(title) < 1)
    sprintf( title,"43m Pointing Model %s (%s %s %s)", 
	     chars[band-1], sourceName, &dateStr[2], timeStr);

  fprintf( stderr, "Plotting pattern %s %s; %ld of %ld (n points= %ld)\n", 
	   &dateStr[2], timeStr,
	   iPattern, nPattern, n);

  sprintf( labelX, 
	  "Sample (%.2lf +/-%.3lf, FWHM=%.1lf; dHa=%.2lf, dDec=%.2lf)",
	  fits[band-1].max.fl[band-1], fits[band-1].rms, 
	  fits[band-1].bmaj/ARCMIN,
	  (fits[band-1].max.daz-pattern.d[n/2].mHa)/ARCMIN,
	  (fits[band-1].max.del-pattern.d[n/2].mDec)/ARCMIN);
  sprintf( labelY, "Intensity (T_SYS fraction)");

  sprintf( outName, "%s_%s", groupName, chars[band-1]);

  if (peaks[band] > 0. && peaks[band] < 20.) 
    sprintf( labelB, "Fit: %.3lf", fits[band-1].max.fl[band-1]);
  else
    strcpy( labelB, "Fit");

  if (doWeb && dataVariation) { /* plot 2-D intensity vs sample with model */
    quickPlot( outName, FALSE, gnuCmds, title,
	     labelX, labelY,
	     n,  haS, 
	     &fluxS[0][0], freqLabels[band-1],
	     &fluxS[1][0], labelB, 
	     NULL, NULL, NULL, NULL);
  }

  sprintf( outName,"pattern3D_%s", chars[band-1]);

  if (doWeb && dataVariation) { /* 3-D of intensity vs position */
    gridPlot( outName, FALSE, "", title, freqLabels[band-1],
	    "HA Offset (arcmin)", "Dec Offset (arcmin)", 
	    "                          Intensity (Tsys Fraction)",
	    NGRIDX,  gridName);

    writeHtml3D( "pattern", chars[band-1], sourceName, "pattern",
		 timeStr);
  } /* end of 3-D plotting */

  sprintf(labelY, 
	  "%8.2lf +/-%7.2lf, FWHM = %7.1lf'; dHa=%7.2lf', dDec=%7.2lf'",
	  fits[band-1].max.fl[band-1], fits[band-1].rms, 
	  fits[band-1].bmaj/ARCMIN,
	  fits[band-1].max.daz/ARCMIN,
	  fits[band-1].max.del/ARCMIN);

  if (doWeb) /* write out pattern file */
    writeHtmlFile( "pattern", chars[band-1], sourceName, labelY, timeStr,
		   FALSE);
  
  if (doWriteFit && (nTSysH > 2) && (nTSysV > 2)) {
     sprintf( title,"T Sys vs Frequency (%s %s %s)", 
	      sourceName, &dateStr[2], timeStr);

     fourPlot("tSysPlot", FALSE, gnuCmds, title,
	       "Frequency (MHz)", "System Temperature (K)",
	       nTSysV, freqVs, tSysVs, "V Polarization", 
	       nTSysH, freqHs, tSysHs, "H Polarization",
	       0, NULL, NULL, NULL,  0, NULL, NULL, NULL);
  }
  
  return(0);
} /* end of pat2Off() */

#define PEAKFREQMHZ  912.5        /* 43m beam width measurement frequency */
#define PEAKWIDTH     34.5        /* 43m beam width in Arcminutes */
void fitPattern( int band, struct pattern * ppat, struct fit * pfit)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* fitPattern() fits a gaussian model to one band of a pattern               */
/* INPUT band      band to be fit (range 0-3)                                */
/*       pattern   measurement data to be fit (all bands)                    */
/* OUTPUT pfit     pointer to fit where results will be placed               */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ int ia[10] = { 1, 1, 1, 1, 1,  1, 1, 1, 1, 1};
  long i = 0;
  double weightAz = 0, weightEl = 0, weight = 0, flux = 0, freq = 0,
    beamWidth = 0;
                                         /* record offset from position */
  for (i = 0; i < ppat->n; i++) {
    flux = ppat->d[i].fl[band];
    if ((ppat->d[i].daz < 30.*ARCMIN) && (ppat->d[i].daz > -30.*ARCMIN) && 
        (ppat->d[i].del < 30.*ARCMIN) && (ppat->d[i].del > -30.*ARCMIN)) {
      weightAz += (flux*(ppat->d[i].daz));
      weightEl += (flux*(ppat->d[i].del));
      weight   += flux;
    }
    if (i % 1000 == 0 && weight > 0)
      fprintf( stderr, "%4ld: %7.2lf %7.2lf %7.3lf: %7.2lf %7.3lf\n",i,
	       ppat->d[i].daz/ARCMIN, ppat->d[i].del/ARCMIN, flux,
	       weightAz/(ARCMIN*weight),weightEl/(ARCMIN*weight));
  }

  if (weight != 0.0) {
    weightAz = weightAz/weight;
    weightEl = weightEl/weight;
  }
  pfit->max.daz=weightAz;
  pfit->max.del=weightEl;

  fprintf( stderr, "Band: %d: Ha, Dec = %7.2lf,%7.2lf\n", band,
	   weightAz/ARCMIN, weightEl/ARCMIN);

  beamWidth = PEAKWIDTH;
  freq = frequencies[band];     
  if (freq > 0.) 
    beamWidth = PEAKWIDTH * PEAKFREQMHZ / freq;
  fprintf( stderr, "Peak model width: %7.2lf' at %8.3lf MHz\n",
	   beamWidth, freq);
  pfit->bmaj = beamWidth * ARCMIN;
  pfit->bmin = beamWidth * ARCMIN;
  pfit->max.fl[band] = 1.;

  printf("At %le,%le, best fit %le,%le,%le\n",
    ppat->az, ppat->el, pfit->max.daz, pfit->max.del, pfit->max.fl[band]);

  ia[WIDTH] = 0;                         /* do not fit width */

  fitGaussNonLin( band, pfit, ia, ppat); /* do fit work */

                                     /* record offset from position */
  pfit->max.az = ppat->az;
  pfit->max.el = ppat->el;
} /* end of fitPattern() */

char * writeFit( char * fitFileName, struct fit * pfit, char * label, char *
	      sourceName)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* writeFit() appends an ascii text string describing the pointing fit to    */
/* the output file.  If the file is not open, writeFit() opens it and adds   */
/* a label string                                                            */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ static FILE *fitfile;
  static int fileOpen = FALSE;
  char fitmsg[255];

  if (fileOpen == FALSE) {           /* if file not open, open it */
    if ((fitfile = fopen( fitFileName,"a+")) == NULL) {
      printf ("writeFit: File not write-able \"%s\"", fitFileName);
      return("Can not Write Output file");
    }
    else {
      fileOpen = TRUE;
    }
    sprintf(fitmsg,
    "#  HA  Declination   LST    Delta X  Delta Y   FluxF1  FluxF2 \
Bmaj F1 Bmaj F2 %s\n", label);
  fprintf(fitfile,"%s",fitmsg);
  } /* end if time to open file */
                                         /* record offset from position */
  if (pfit->max.az < 0) 
    pfit->max.az += TWOPI;

  sprintf(fitmsg,
    "%8.5lf %8.5lf %8.6lf %8.3lf %8.3lf %7.2lf %7.2lf %6.1lf %6.1lf %s\n",
    pfit->max.az, pfit->max.el, pfit->bpa, 
    pfit->max.daz/ARCMIN, pfit->max.del/ARCMIN, 
    pfit->max.fl[0], pfit->max.fl[2], pfit->bmaj/ARCMIN, pfit->bmin/ARCMIN,
	  sourceName);
  printf("%s",fitmsg);
  fprintf(fitfile,"%s",fitmsg);
  return(NULL);
} /* end of writeFit() */

char * writeOffs( char * offFileName, struct fit * poff, double azOffs[], 
		double elOffs[], char * sourceName)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* writeOffs() appends an ascii text string describing the pointing Offsets  */
/* to the output file.  If the file is not open, writeOffs() opens it and    */
/* adds a label string.  The offset entry contains the measured offset from  */
/* the nominal source position for each of the four bands of data.           */
/* The beam squint between the two circular polarization can be calcuated    */
/* from the output measurements.  Also the alignment between X and Ku can be */
/* determined.                                                               */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ static FILE *offfile;
  static int fileOpen = FALSE;
  char offmsg[255];

  if (fileOpen == FALSE) {
    if ((offfile = fopen( offFileName,"a+")) == NULL) {
      printf ("writeOff: File not write-able \"%s\"", offFileName);
      return("Can Not Write Output File");
    }
    else 
      fileOpen = TRUE;
    /* on first file open, add value labels */
    sprintf(offmsg,
    "#  HA  Declination   LST    \
Dx F1  Dy F1  Dx F2  Dy F2  Dx F3  Dy F3  Dx F4  Dy F4\n");
  fprintf(offfile,"%s",offmsg);
  } /* end if time to open file */

  if (poff->max.az < 0) 
    poff->max.az += TWOPI;
                                         /* record offset from position */
  sprintf(offmsg,
"%8.5lf %8.5lf %8.6lf %6.1lf %6.1lf %6.1lf %6.1lf %6.1lf %6.1lf %6.1lf %6.1lf %s\n",
    poff->max.az, poff->max.el, poff->bpa, 
	  azOffs[0]/ARCMIN, elOffs[0]/ARCMIN, 
	  azOffs[1]/ARCMIN, elOffs[1]/ARCMIN,
	  azOffs[2]/ARCMIN, elOffs[2]/ARCMIN,
	  azOffs[3]/ARCMIN, elOffs[3]/ARCMIN,
	  sourceName);
  printf("%s",offmsg);
  fprintf(offfile,"%s",offmsg);
  return(NULL);
} /* end of writeOff */

void writePeak( char * peakFileName, struct fit * ppeak, double peaks[])
/* output a measured fit to filename */
{ static FILE *peakfile;
  static int fileOpen = FALSE;
  char peakmsg[255];

  if (fileOpen == FALSE) {
    if ((peakfile = fopen( peakFileName,"a+")) == NULL) {
      printf ("writePeak: File not write-able \"%s\"", peakFileName);
      return;
    }
    else {
      fileOpen = TRUE;
    }
    sprintf(peakmsg,
    "#  HA  Declination   LST     Max F1  Max F2  Max F3  Max F4\n");
    fprintf(peakfile,"%s",peakmsg);
  } /* end if time to open file */
                                         /* record peakset from position */
  sprintf(peakmsg, "%8.5lf %8.5lf %8.6lf %7.2lf %7.2lf %7.2lf %7.2lf\n",
    ppeak->max.az, ppeak->max.el, ppeak->bpa, 
	  peaks[0], peaks[1], peaks[2], peaks[3]); 
  printf("%s",peakmsg);
  fprintf(peakfile,"%s",peakmsg);
} /* end of writePeak */


