/* File quick3d.c, version 1.3, released  09/04/07 at 20:34:53 
   retrieved by SC 09/05/19 at 16:30:29     

%% 3-d data plotting routines

:: Offline 

HISTORY:
 090407 GIL allow plotting coordiantes on x,y axies
 090403 GIL routines to produce 3d plots of x-y arrays of data

DESCRIPTION:
These functions gather data for gridding and plotting using gnuplot's
3-d data mode
*/

#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "MATHCNST.H"
#include "STDDEFS.H"

/* externals */
extern char * angleLimit( double * inOutAngle);
extern char * angleRange( double minAngle, double maxAngle, double * angle);
extern char * gridPlot( char * plotName, long gv, 
		 char * gnuCmds, char * title, char * object,
			char * xLabel,  char * yLabel, char * zLabel,
		 long n,  char * gridFileName);
extern char *stripWhite(char *);  /* remove leading and trailing blanks */
extern char *str2rad ( char *, double *);/* read double from string */
extern char * extractName(char * fullPath, char * fileName, char * extension);
                                    /* Fit measurements to model */
extern char * fourPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n1, double * x1s, double * y1s, char * y1Label,
		  long n2, double * x2s, double * y2s, char * y2Label,
		  long n3, double * x3s, double * y3s, char * y3Label,
		 long n4, double * x4s, double * y4s, char * y4Label);
extern char * quickPlot( char * plotName, long gv, 
		  char * gnuCmds, char * title, 
		  char * xLabel,  char * yLabel, 
		  long n,  double * xs, 
		  double * y1s, char * y1Label,
		  double * y2s, char * y2Label,
                  double * y3s, char * y3Label,
		  double * y4s, char * y4Label);
extern char * cvrtuc( char *);
extern char * mjd2str( long, char *);
extern char * rad2str( double, char *, char *);
extern double median4( double a, double b, double c, double d);
extern double now2Utc();
extern long today2mjd();
extern double medianSelect(unsigned long k, unsigned long n, double arr[]);
char * calcAveRmsRange( double a[], long aStart, long aEnd, 
			long bStart, long bEnd, 
			double * aveOut, double * rmsOut);

/* internals */

#define VERSIONSTR "1.0"
#define EPSILON    ARCSEC

#define NGRIDX 1025           /* define the 3-d grid */
#define NGRIDY 1025

double map[NGRIDX][NGRIDY], weight[NGRIDX][NGRIDY];

/* convolving kernal half size in cells */
long nX = NGRIDX, nY = NGRIDY, iX0 = NGRIDX/2, iY0 = NGRIDY/2, nIn = 0;
double nConvolve = 2.;

char * initGrid(long inX, long inY)        /* initialize counts and 
					     grid values */
{ long i = 0, j = 0;
  if ((inX <= NGRIDX) && (inX > 0))        /* if values in valid range, use */
    nX = inX;
  if ((inY <= NGRIDY) && (inY > 0))
    nY = inY;

  iX0 = nX/2;                              /* define the reference pixel */
  iY0 = nY/2;

  for (i = 0; i < NGRIDX; i++) {           /* init map and weighs */
    for (j = 0; j < NGRIDY; j++) {
      map[i][j] = weight[i][j] = 0.;
    }
  }
  nIn = 0;                                  /* init point count */
  return(NULL);
} /* end of initGrid() */

char * gridXYZ( double x0, double y0, double dX0, double dY0, long n, 
	       double xs[], double ys[], double zs[])
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* gridXYZ() convolves z values onto a grid.                                 */
/* INPUTS  x0, y0 coordinate of the reference pixel (defined at init) */
/*         dX, dY size of pixels in xy units                          */
/*         n      number of input values in this set of data arrays   */
/*  xs,ys,zs      x,y,z arrays, each of length n (range 0 to n-1)     */
/* OUTPUTS  global arrays map[][] and weight[][] containing values    */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char * errMsg = NULL;
  long i = 0, j = 0, k = 0, iX = 0, jY = 0;
  double dR2 = 0., x = 0, y = 0, dX = 0, dY = 0, w = 0., r2 = 0;

  if (dX0 == 0 || dY0 == 0) {
    fprintf( stderr, "grid sizes invalid dX,dY = %lg,%lg\n", dX0, dY0);
    errMsg = "Invalid Grid Size";
    return(errMsg);
  }
  r2 = (dX0*dX0)+(dY0*dY0);
  if (r2 <= 0.)
    return("Invalid pixel scale");

  for (k = 0; k < n; k++) {            /* for all points in pattern */
    x = xs[k] - x0;                    /* compute x offset relative to center*/
    y = ys[k] - y0;
    iX = x/dX0 + iX0;                  /* comput pixel offset */
    jY = y/dY0 + iY0;
    if (k%200 == 0)
      fprintf( stderr, "Point %4ld: %8.3lf,%8.3lf,%8.3lf => %4ld,%4ld\n",
	       k, x, y, zs[k], iX, jY);

    for (i = iX-nConvolve; i < iX+nConvolve+1; i++) {
      if (i >= 0 && i < nX) {
	dX = (i - iX);
	for (j = jY-nConvolve; j < jY+nConvolve+1; j++) {
	  if (j >= 0 && j < nY) {
	    dY = (j - jY);
	    /* compute radius squared in pixels */
	    dR2 = ((dX*dX) + (dY*dY));
	    if (dR2 > 0.)
	      w = exp( - (double)nConvolve/dR2);
	    else
	      w = 1;
	    map[i][j] = map[i][j] + (zs[k] * w);
	    weight[i][j] = weight[i][j] + w;
	    nIn ++;
	  } /* end if j on grid */
	} /* end for all j in convolving function */
      } /* end if i on grid */
    } /* end for all i in convolving function */
  } /* end for all points in pattern */

  return(errMsg);
} /* end of gridXYZ() */

char * normalizeGrid()
{ char * errMsg = NULL;
  long i = 0, j = 0;

  for (i = 0; i < nX; i++) { /* now normalize the grid */
    for (j = 0; j < nY; j++) {
      if (weight[i][j] > 0.) 
	map[i][j] = map[i][j]/weight[i][j];
    } /* end for all j */
  } /* end for all i */
  return(errMsg);
} /* end of normalizeGrid() */

char * minMaxGrid( double * minZ, double * maxZ)
{ char * errMsg = NULL;
  long iMin = 0, jMin = 0, iMax = 0, jMax = 0, i = 0, j = 0;

  for (i = 0; i < nX; i++) { /* now normalize the grid */
    for (j = 0; j < nY; j++) {
      if (map[i][j] < map[iMin][jMin]) {
	iMin = i; 
	jMin = j;
      }
      else if (map[i][j] > map[iMax][jMax]) {
	iMax = i; 
	jMax = j;
      }
    } /* end for all j */
  } /* end for all i */
  
  *minZ = map[iMin][jMin];
  *maxZ = map[iMax][jMax];
  return(errMsg);
} /* end of minMaxGrid() */

char * writeHtml3D( char * groupName, char * plotName, 
		      char * labelA, char * labelB, char * timeStr) 
{ char systemStr[1000] = "", htmlFileName[1000] = "";
  FILE * pF = NULL;

  strcpy( htmlFileName, groupName);
  strcat( htmlFileName, "3D_");
  strcat( htmlFileName, plotName);
  strcat( htmlFileName, ".htm");
  if ((pF = fopen( htmlFileName,"w+")) == NULL) {  /* open output file */
    printf ("writeHtml3D: HTML file not write-able \"%s\"", htmlFileName);
    return("Error writing the html file");
  }

  fprintf( pF, "<html><head>\n");

  /* create file header, indicating file should be refreshed */
  fprintf( pF, "<meta http-equiv=\"refresh\" content=\"60\">\n");
  fprintf( pF, 
	   "<title>%s</title>\n", groupName);
  fprintf( pF, 
	   "<body text=\"#000000\" bgcolor=\"#ffffff\" ");
  fprintf( pF, 
	   "link=\"#880066\" vlink=\"#0000cc\"\n");
  fprintf( pF, "</head><body>\n");
  fprintf( pF, "<CENTER><FONT FACE=helvetica SIZE=\"+2\">\n");
  fprintf( pF, "%s %s\n", labelA, labelB);
  fprintf( pF, "</font></CENTER>\n");
  fprintf( pF, "<center>\n");
  fprintf( pF, "<a href=\"%s3D_%s.ps\">", groupName, plotName);
  fprintf( pF, "<img src=\"%s3D_%s.gif\"</a>", groupName, plotName);
  fprintf( pF, "</center>\n");
  fclose( pF); 

  sprintf( systemStr, "chmod 666 %s", htmlFileName);
  system(systemStr);  

  return(NULL);
} /* end of writeHtml3D */

char * writeGrid( char * fileName)
{ char * errMsg = NULL;
  long i = 0, j = 0;
  FILE * gFile = NULL;

  gFile = fopen ( fileName, "w+");
  if ( gFile == NULL) {
    fprintf( stderr, "Error opening output grid file: %s\n", fileName);
    return( "Error opening grid file");
  }

  for (j = 0; j < nY; j++) {
    for (i = 0; i < nX; i++) {          /* write all points in grid */
      fprintf( gFile, "%.3lf ", map[i][j]); 
    } /* end for all j */
    fprintf( gFile, "\n");
  } /* end for all i */
  fclose(gFile);                          /* important! close file or no plot*/

  return(errMsg);
} /* end of writeGrid() */

char * writeGrid2( char * fileName, double gridXs[], double gridYs[])
{ char * errMsg = NULL;
  long i = 0, j = 0;
  FILE * gFile = NULL;

  gFile = fopen ( fileName, "w+");
  if ( gFile == NULL) {
    fprintf( stderr, "Error opening output grid file: %s\n", fileName);
    return( "Error opening grid file");
  }

  for (j = 0; j < nY; j++) {
    for (i = 0; i < nX; i++) {          /* write all points in grid */
    fprintf( gFile, "%.2lf ", gridXs[j]);
      fprintf( gFile, "%.2lf ", gridYs[i]);
      fprintf( gFile, "%.3lf ", map[i][j]); 
      fprintf( gFile, "\n");
    } /* end for all j */
    fprintf( gFile, "\n");
  } /* end for all i */
  fclose(gFile);                          /* important! close file or no plot*/

  return(errMsg);
} /* end of writeGrid() */

char * clipGrid( double minIn, double maxIn)
/* clipGrid() clips the grid points to between min and max values */
{ long i = 0, j = 0;

  for (i = 0; i < nX; i++) {          /* init map and weighs */
    for (j = 0; j < nY; j++) {
      if (map[i][j] < minIn) 
	map[i][j] = minIn;
      else if (map[i][j] > maxIn) 
	map[i][j] = maxIn;
    }
  } /* end for all i */

  return(NULL);
} /* end of clipGrid() */

char * readXYZs( char * fileName, long nMax, 
		 long * nOut, double xs[], double ys[], double zs[])
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* readXYZs() takes an input file name, opens, reads and closes the file     */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char * errMsg = NULL, record[501] = "";
  long n = 0;
  double x = 0, y = 0, z = 0;
  FILE * pFile = NULL;

  if ((pFile = fopen( fileName, "r")) == NULL) {
    fprintf( stderr, "readXYZs: Error opening file: %s\n", fileName);
    return("Error opening model file");
  }

  do {                  
    if (fgets(record,500,pFile) == NULL)   /* if end of file, exit */
      break;
    stripWhite( record);                
    if (strlen(record) < 3)             /* ignore short lines */
      continue;

    /* if a comment, ignore */
    if (record[0] == '#' || record[0] == '!' || record[0] == '%')  
      continue;
    /* if here, this must be data! */
    sscanf( record, "%lg %lg %lg", &x, &y, &z);
    xs[n] = x;
    ys[n] = y;
    zs[n] = z;
    n++;
  } while (n < nMax); 

  *nOut = n;
  fclose ( pFile);

  fprintf( stderr, "Read %5ld values from file: %s\n", *nOut, fileName);

  return(errMsg);
} /* end of readXYZs() */

#define NVALUES 10000

char * quick3d( char * xyzFileName)
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
/* fullGrid() is an example of full use of the gridding functions            */
/* INPUT patternFileName     name of file containing raw pattern measurements*/
/*       dateString          extenstion added to output files                */
/*  *    *    *    *    *    *    *    *    *    *    *    *    *    *    *  */
{ char outFileName[1000] = "", * errMsg = NULL,
    fullPath[1000] = "", fileName[1000] = "", extension[100] = "",
    gridName[1000] = "/tmp/gridPattern.txt", 
    title[132] = "";
  long n = 0, nMax = NVALUES, mapN = 200;
  double minZ = 0, maxZ = 0, xs[NVALUES], ys[NVALUES], zs[NVALUES], 
    xAve = 0, xRms = 0, yAve = 0, yRms = 0, zAve = 0, zRms = 0,
    dX0 = 1, dY0 = 1;

  strcpy( fullPath, xyzFileName);
  extractName( fullPath, fileName, extension);
  strcpy( title, fileName);

  errMsg = readXYZs( fullPath, nMax, &n, xs, ys, zs);
  if (errMsg)
    return(errMsg);
  if (n < 3) {
    fprintf ( stderr, "quick3d: file %s; not enough data: %ld\n",
	      fileName, n);
    return( "Not Enough Data");
  }

  errMsg = initGrid( mapN, mapN);

  calcAveRmsRange( xs, 1, n, n, n, &xAve, &xRms);
  calcAveRmsRange( ys, 1, n, n, n, &yAve, &yRms);
  calcAveRmsRange( zs, 1, n, n, n, &zAve, &zRms);

  printf( "X ave %8.3lf, rms %8.3lf\n", xAve, xRms);
  printf( "Y ave %8.3lf, rms %8.3lf\n", yAve, yRms);
  printf( "Y ave %8.3lf, rms %8.3lf\n", zAve, zRms);

  dX0 = 6*xRms/(double)mapN;
  if (dX0 == 0)
    dX0 = 1;
  dY0 = 6*yRms/(double)mapN;
  if (dY0 == 0)
    dY0 = 1;
  errMsg = gridXYZ( xAve, yAve, dX0, dY0, n, xs, ys, zs);
  if (errMsg)
    fprintf( stderr, "%s\n", errMsg);

  normalizeGrid();
  minMaxGrid( &minZ, &maxZ);
  fprintf( stderr, "Z grid range: %lg to %lg\n", minZ, maxZ);
  if (minZ < (zAve - 3.*zRms))
    minZ = (zAve - 3.*zRms);
  if (maxZ < (zAve + 20.*zRms))
    maxZ = (zAve + 20.*zRms);

  clipGrid( minZ, maxZ);

  if (minZ >= maxZ) {
    fprintf( stderr, "Error finding data range: %lg !< %lg\n",
	     minZ, maxZ);
    return("Data range error");
  }

  remove( gridName);
  errMsg = writeGrid2( gridName, xs, ys);

  strcpy( outFileName, "/tmp/");
  strcat( outFileName, fileName);
  gridPlot( outFileName, TRUE, "", title, "",
	    "  X  ", "  Y  ",  
	    "             Intensity    ",
	    nX,  gridName);

  return(errMsg);
} /* end of quick3d() */
